@@ -4,15 +4,17 @@ __copyright__ = '2008, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 import sys, os, re, logging, time, mimetypes, \
        __builtin__, warnings, multiprocessing
+from urllib import getproxies
 __builtin__.__dict__['dynamic_property'] = lambda(func): func(None)
 from htmlentitydefs import name2codepoint
 from math import floor
+from functools import partial
 
 warnings.simplefilter('ignore', DeprecationWarning)
 
 
 from calibre.startup import plugins, winutil, winutilerror
-from calibre.constants import iswindows, isosx, islinux, isfrozen, \
+from calibre.constants import iswindows, isosx, islinux, isfreebsd, isfrozen, \
                               terminal_controller, preferred_encoding, \
                               __appname__, __version__, __author__, \
                               win32event, win32api, winerror, fcntl, \
@@ -22,12 +24,13 @@ import mechanize
 if False:
     winutil, winutilerror, __appname__, islinux, __version__
     fcntl, win32event, isfrozen, __author__, terminal_controller
-    winerror, win32api
+    winerror, win32api, isfreebsd
 
 mimetypes.add_type('application/epub+zip',                '.epub')
 mimetypes.add_type('text/x-sony-bbeb+xml',                '.lrs')
 mimetypes.add_type('application/xhtml+xml',               '.xhtml')
 mimetypes.add_type('image/svg+xml',                       '.svg')
+mimetypes.add_type('text/fb2+xml',                        '.fb2')
 mimetypes.add_type('application/x-sony-bbeb',             '.lrf')
 mimetypes.add_type('application/x-sony-bbeb',             '.lrx')
 mimetypes.add_type('application/x-dtbncx+xml',            '.ncx')
@@ -35,10 +38,13 @@ mimetypes.add_type('application/adobe-page-template+xml', '.xpgt')
 mimetypes.add_type('application/x-font-opentype',         '.otf')
 mimetypes.add_type('application/x-font-truetype',         '.ttf')
 mimetypes.add_type('application/oebps-package+xml',       '.opf')
-mimetypes.add_type('application/ereader',                 '.pdb')
+mimetypes.add_type('application/vnd.palm',                '.pdb')
 mimetypes.add_type('application/x-mobipocket-ebook',      '.mobi')
 mimetypes.add_type('application/x-mobipocket-ebook',      '.prc')
 mimetypes.add_type('application/x-mobipocket-ebook',      '.azw')
+mimetypes.add_type('application/x-cbz',                   '.cbz')
+mimetypes.add_type('application/x-cbr',                   '.cbr')
+mimetypes.add_type('application/x-koboreader-ebook',      '.kobo')
 mimetypes.add_type('image/wmf',                           '.wmf')
 guess_type = mimetypes.guess_type
 import cssutils
@@ -139,7 +145,10 @@ def prints(*args, **kwargs):
                             raise
                         arg = repr(arg)
 
-        file.write(arg)
+        try:
+            file.write(arg)
+        except:
+            file.write(repr(arg))
         if i != len(args)-1:
             file.write(sep)
     file.write(end)
@@ -198,71 +207,54 @@ def extract(path, dir):
     extractor(path, dir)
 
 def get_proxies(debug=True):
-    proxies = {}
+    proxies = getproxies()
+    for key, proxy in list(proxies.items()):
+        if not proxy or '..' in proxy:
+            del proxies[key]
+            continue
+        if proxy.startswith(key+'://'):
+            proxy = proxy[len(key)+3:]
+        if proxy.endswith('/'):
+            proxy = proxy[:-1]
+        if len(proxy) > 4:
+            proxies[key] = proxy
+        else:
+            prints('Removing invalid', key, 'proxy:', proxy)
+            del proxies[key]
 
-    for q in ('http', 'ftp'):
-        proxy =  os.environ.get(q+'_proxy', None)
-        if not proxy: continue
-        if proxy.startswith(q+'://'):
-            proxy = proxy[7:]
-        proxies[q] = proxy
-
-    if iswindows:
-        try:
-            winreg = __import__('_winreg')
-            settings = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
-                                      'Software\\Microsoft\\Windows'
-                                      '\\CurrentVersion\\Internet Settings')
-            proxy = winreg.QueryValueEx(settings, "ProxyEnable")[0]
-            if proxy:
-                server = str(winreg.QueryValueEx(settings, 'ProxyServer')[0])
-                if ';' in server:
-                    for p in server.split(';'):
-                        protocol, address = p.split('=')
-                        proxies[protocol] = address
-                else:
-                    proxies['http'] = server
-                    proxies['ftp'] =  server
-            settings.Close()
-        except Exception, e:
-            prints('Unable to detect proxy settings: %s' % str(e))
-    for x in list(proxies):
-        if len(proxies[x]) < 5:
-            prints('Removing invalid', x, 'proxy:', proxies[x])
-            del proxies[x]
     if proxies and debug:
         prints('Using proxies:', proxies)
     return proxies
 
 def get_parsed_proxy(typ='http', debug=True):
     proxies = get_proxies(debug)
-    if typ not in proxies:
-        return
-    pattern = re.compile((
-        '(?:ptype://)?' \
-        '(?:(?P<user>\w+):(?P<pass>.*)@)?' \
-        '(?P<host>[\w\-\.]+)' \
-        '(?::(?P<port>\d+))?').replace('ptype', typ)
-    )
+    proxy = proxies.get(typ, None)
+    if proxy:
+        pattern = re.compile((
+            '(?:ptype://)?' \
+            '(?:(?P<user>\w+):(?P<pass>.*)@)?' \
+            '(?P<host>[\w\-\.]+)' \
+            '(?::(?P<port>\d+))?').replace('ptype', typ)
+        )
 
-    match = pattern.match(proxies['typ'])
-    if match:
-        try:
-            ans = {
-                    'host' : match.group('host'),
-                    'port' : match.group('port'),
-                    'user' : match.group('user'),
-                    'pass' : match.group('pass')
-                }
-            if ans['port']:
-                ans['port'] = int(ans['port'])
-        except:
-            if debug:
-                traceback.print_exc()
-            return
-        if debug:
-            prints('Using http proxy', ans)
-        return ans
+        match = pattern.match(proxies[typ])
+        if match:
+            try:
+                ans = {
+                        'host' : match.group('host'),
+                        'port' : match.group('port'),
+                        'user' : match.group('user'),
+                        'pass' : match.group('pass')
+                    }
+                if ans['port']:
+                    ans['port'] = int(ans['port'])
+            except:
+                if debug:
+                    traceback.print_exc()
+            else:
+                if debug:
+                    prints('Using http proxy', str(ans))
+                return ans
 
 
 def browser(honor_time=True, max_time=2, mobile_browser=False):
@@ -350,13 +342,6 @@ def detect_ncpus():
     return ans
 
 
-def launch(path_or_url):
-    from PyQt4.QtCore import QUrl
-    from PyQt4.QtGui  import QDesktopServices
-    if os.path.exists(path_or_url):
-        path_or_url = 'file:'+path_or_url
-    QDesktopServices.openUrl(QUrl(path_or_url))
-
 relpath = os.path.relpath
 _spat = re.compile(r'^the\s+|^a\s+|^an\s+', re.IGNORECASE)
 def english_sort(x, y):
@@ -399,42 +384,59 @@ def my_unichr(num):
     except ValueError:
         return u'?'
 
-def entity_to_unicode(match, exceptions=[], encoding='cp1252'):
+def entity_to_unicode(match, exceptions=[], encoding='cp1252',
+        result_exceptions={}):
     '''
-    @param match: A match object such that '&'+match.group(1)';' is the entity.
-    @param exceptions: A list of entities to not convert (Each entry is the name of the entity, for e.g. 'apos' or '#1234'
-    @param encoding: The encoding to use to decode numeric entities between 128 and 256.
+    :param match: A match object such that '&'+match.group(1)';' is the entity.
+
+    :param exceptions: A list of entities to not convert (Each entry is the name of the entity, for e.g. 'apos' or '#1234'
+
+    :param encoding: The encoding to use to decode numeric entities between 128 and 256.
     If None, the Unicode UCS encoding is used. A common encoding is cp1252.
+
+    :param result_exceptions: A mapping of characters to entities. If the result
+    is in result_exceptions, result_exception[result] is returned instead.
+    Convenient way to specify exception for things like < or > that can be
+    specified by various actual entities.
     '''
+    def check(ch):
+        return result_exceptions.get(ch, ch)
+
     ent = match.group(1)
     if ent in exceptions:
         return '&'+ent+';'
     if ent == 'apos':
-        return "'"
+        return check("'")
     if ent == 'hellips':
         ent = 'hellip'
-    if ent.startswith(u'#x'):
+    if ent.lower().startswith(u'#x'):
         num = int(ent[2:], 16)
         if encoding is None or num > 255:
-            return my_unichr(num)
-        return chr(num).decode(encoding)
+            return check(my_unichr(num))
+        return check(chr(num).decode(encoding))
     if ent.startswith(u'#'):
         try:
             num = int(ent[1:])
         except ValueError:
             return '&'+ent+';'
         if encoding is None or num > 255:
-            return my_unichr(num)
+            return check(my_unichr(num))
         try:
-            return chr(num).decode(encoding)
+            return check(chr(num).decode(encoding))
         except UnicodeDecodeError:
-            return my_unichr(num)
+            return check(my_unichr(num))
     try:
-        return my_unichr(name2codepoint[ent])
+        return check(my_unichr(name2codepoint[ent]))
     except KeyError:
         return '&'+ent+';'
 
 _ent_pat = re.compile(r'&(\S+?);')
+xml_entity_to_unicode = partial(entity_to_unicode, result_exceptions = {
+    '"' : '&quot;',
+    "'" : '&apos;',
+    '<' : '&lt;',
+    '>' : '&gt;',
+    '&' : '&amp;'})
 
 def prepare_string_for_xml(raw, attribute=False):
     raw = _ent_pat.sub(entity_to_unicode, raw)
@@ -443,6 +445,23 @@ def prepare_string_for_xml(raw, attribute=False):
         raw = raw.replace('"', '&quot;').replace("'", '&apos;')
     return raw
 
+def isbytestring(obj):
+    return isinstance(obj, (str, bytes))
+
+def human_readable(size):
+    """ Convert a size in bytes into a human readable form """
+    divisor, suffix = 1, "B"
+    for i, candidate in enumerate(('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB')):
+        if size < 1024**(i+1):
+            divisor, suffix = 1024**(i), candidate
+            break
+    size = str(float(size)/divisor)
+    if size.find(".") > -1:
+        size = size[:size.find(".")+2]
+    if size.endswith('.0'):
+        size = size[:-2]
+    return size + " " + suffix
+
 if isosx:
     import glob, shutil
     fdir = os.path.expanduser('~/.fonts')
