@@ -7,7 +7,7 @@ import os
 import glob
 from calibre.customize import FileTypePlugin, MetadataReaderPlugin, MetadataWriterPlugin
 from calibre.constants import numeric_version
-from calibre.ebooks.metadata.archive import ArchiveExtract
+from calibre.ebooks.metadata.archive import ArchiveExtract, get_cbz_metadata
 
 class HTML2ZIP(FileTypePlugin):
     name = 'HTML to ZIP'
@@ -30,6 +30,7 @@ every time you add an HTML file to the library.\
 
         with TemporaryDirectory('_plugin_html2zip') as tdir:
             recs =[('debug_pipeline', tdir, OptionRecommendation.HIGH)]
+            recs.append(['keep_ligatures', True, OptionRecommendation.HIGH])
             if self.site_customization and self.site_customization.strip():
                 recs.append(['input_encoding', self.site_customization.strip(),
                     OptionRecommendation.HIGH])
@@ -68,11 +69,12 @@ class PML2PMLZ(FileTypePlugin):
 
         of = self.temporary_file('_plugin_pml2pmlz.pmlz')
         pmlz = zipfile.ZipFile(of.name, 'w')
-        pmlz.write(pmlfile, os.path.basename(pmlfile))
+        pmlz.write(pmlfile, os.path.basename(pmlfile), zipfile.ZIP_DEFLATED)
 
-        pml_img = os.path.basename(pmlfile)[0] + '_img'
-        img_dir = pml_img if os.path.exists(pml_img) else 'images' if \
-            os.path.exists('images') else ''
+        pml_img = os.path.splitext(pmlfile)[0] + '_img'
+        i_img = os.path.join(os.path.dirname(pmlfile),'images')
+        img_dir = pml_img if os.path.isdir(pml_img) else i_img if \
+            os.path.isdir(i_img) else ''
         if img_dir:
             for image in glob.glob(os.path.join(img_dir, '*.png')):
                 pmlz.write(image, os.path.join('images', (os.path.basename(image))))
@@ -80,7 +82,7 @@ class PML2PMLZ(FileTypePlugin):
 
         return of.name
 
-
+# Metadata reader plugins {{{
 class ComicMetadataReader(MetadataReaderPlugin):
 
     name = 'Read comic metadata'
@@ -96,12 +98,29 @@ class ComicMetadataReader(MetadataReaderPlugin):
         from calibre.ebooks.metadata import MetaInformation
         ret = extract_first(stream)
         mi = MetaInformation(None, None)
+        stream.seek(0)
+        if ftype == 'cbz':
+            try:
+                mi.smart_update(get_cbz_metadata(stream))
+            except:
+                pass
         if ret is not None:
             path, data = ret
             ext = os.path.splitext(path)[1][1:]
             mi.cover_data = (ext.lower(), data)
         return mi
 
+class CHMMetadataReader(MetadataReaderPlugin):
+
+    name        = 'Read CHM metadata'
+    file_types  = set(['chm'])
+    description = _('Read metadata from %s files') % 'CHM'
+
+    def get_metadata(self, stream, ftype):
+        from calibre.ebooks.chm.metadata import get_metadata
+        return get_metadata(stream)
+
+
 class EPUBMetadataReader(MetadataReaderPlugin):
 
     name        = 'Read EPUB metadata'
@@ -301,7 +320,9 @@ class ZipMetadataReader(MetadataReaderPlugin):
     def get_metadata(self, stream, ftype):
         from calibre.ebooks.metadata.zip import get_metadata
         return get_metadata(stream)
+# }}}
 
+# Metadata writer plugins {{{
 
 class EPUBMetadataWriter(MetadataWriterPlugin):
 
@@ -311,7 +332,7 @@ class EPUBMetadataWriter(MetadataWriterPlugin):
 
     def set_metadata(self, stream, mi, type):
         from calibre.ebooks.metadata.epub import set_metadata
-        set_metadata(stream, mi)
+        set_metadata(stream, mi, apply_null=self.apply_null)
 
 class LRFMetadataWriter(MetadataWriterPlugin):
 
@@ -366,6 +387,18 @@ class RTFMetadataWriter(MetadataWriterPlugin):
         from calibre.ebooks.metadata.rtf import set_metadata
         set_metadata(stream, mi)
 
+class TOPAZMetadataWriter(MetadataWriterPlugin):
+
+    name        = 'Set TOPAZ metadata'
+    file_types  = set(['tpz', 'azw1'])
+    description = _('Set metadata in %s files')%'TOPAZ'
+    author      = 'Greg Riker'
+
+    def set_metadata(self, stream, mi, type):
+        from calibre.ebooks.metadata.topaz import set_metadata
+        set_metadata(stream, mi)
+
+# }}}
 
 from calibre.ebooks.comic.input import ComicInput
 from calibre.ebooks.epub.input import EPUBInput
@@ -383,6 +416,7 @@ from calibre.ebooks.rtf.input import RTFInput
 from calibre.ebooks.tcr.input import TCRInput
 from calibre.ebooks.txt.input import TXTInput
 from calibre.ebooks.lrf.input import LRFInput
+from calibre.ebooks.chm.input import CHMInput
 
 from calibre.ebooks.epub.output import EPUBOutput
 from calibre.ebooks.fb2.output import FB2Output
@@ -400,31 +434,39 @@ from calibre.ebooks.txt.output import TXTOutput
 
 from calibre.customize.profiles import input_profiles, output_profiles
 
-
+from calibre.devices.apple.driver import ITUNES
 from calibre.devices.hanlin.driver import HANLINV3, HANLINV5, BOOX
 from calibre.devices.blackberry.driver import BLACKBERRY
 from calibre.devices.cybook.driver import CYBOOK
 from calibre.devices.eb600.driver import EB600, COOL_ER, SHINEBOOK, \
                 POCKETBOOK360, GER2, ITALICA, ECLICTO, DBOOK, INVESBOOK, \
-                BOOQ
+                BOOQ, ELONEX, POCKETBOOK301, MENTOR
 from calibre.devices.iliad.driver import ILIAD
 from calibre.devices.irexdr.driver import IREXDR1000, IREXDR800
 from calibre.devices.jetbook.driver import JETBOOK
 from calibre.devices.kindle.driver import KINDLE, KINDLE2, KINDLE_DX
 from calibre.devices.nook.driver import NOOK
-from calibre.devices.prs500.driver import PRS500
-from calibre.devices.prs505.driver import PRS505, PRS700
-from calibre.devices.android.driver import ANDROID
-from calibre.devices.nokia.driver import N770, N810
-from calibre.devices.eslick.driver import ESLICK
+from calibre.devices.prs505.driver import PRS505
+from calibre.devices.android.driver import ANDROID, S60
+from calibre.devices.nokia.driver import N770, N810, E71X, E52
+from calibre.devices.eslick.driver import ESLICK, EBK52
 from calibre.devices.nuut2.driver import NUUT2
 from calibre.devices.iriver.driver import IRIVER_STORY
 from calibre.devices.binatone.driver import README
-from calibre.devices.hanvon.driver import N516, EB511
+from calibre.devices.hanvon.driver import N516, EB511, ALEX, AZBOOKA, THEBOOK
+from calibre.devices.edge.driver import EDGE
+from calibre.devices.teclast.driver import TECLAST_K3, NEWSMY, IPAPYRUS
+from calibre.devices.sne.driver import SNE
+from calibre.devices.misc import PALMPRE, AVANT, SWEEX
+from calibre.devices.folder_device.driver import FOLDER_DEVICE_FOR_CONFIG
+from calibre.devices.kobo.driver import KOBO
 
-from calibre.ebooks.metadata.fetch import GoogleBooks, ISBNDB, Amazon
+from calibre.ebooks.metadata.fetch import GoogleBooks, ISBNDB, Amazon, \
+    LibraryThing
+from calibre.ebooks.metadata.douban import DoubanBooks
 from calibre.library.catalog import CSV_XML, EPUB_MOBI
-plugins = [HTML2ZIP, PML2PMLZ, ArchiveExtract, GoogleBooks, ISBNDB, Amazon, CSV_XML, EPUB_MOBI]
+plugins = [HTML2ZIP, PML2PMLZ, ArchiveExtract, GoogleBooks, ISBNDB, Amazon,
+        LibraryThing, DoubanBooks, CSV_XML, EPUB_MOBI]
 plugins += [
     ComicInput,
     EPUBInput,
@@ -442,6 +484,7 @@ plugins += [
     TCRInput,
     TXTInput,
     LRFInput,
+    CHMInput,
 ]
 plugins += [
     EPUBOutput,
@@ -470,18 +513,21 @@ plugins += [
     JETBOOK,
     SHINEBOOK,
     POCKETBOOK360,
+    POCKETBOOK301,
     KINDLE,
     KINDLE2,
     KINDLE_DX,
     NOOK,
     PRS505,
-    PRS700,
-    PRS500,
     ANDROID,
+    S60,
     N770,
+    E71X,
+    E52,
     N810,
     COOL_ER,
     ESLICK,
+    EBK52,
     NUUT2,
     IRIVER_STORY,
     GER2,
@@ -494,7 +540,23 @@ plugins += [
     EB600,
     README,
     N516,
+    THEBOOK,
     EB511,
+    ELONEX,
+    TECLAST_K3,
+    NEWSMY,
+    IPAPYRUS,
+    EDGE,
+    SNE,
+    ALEX,
+    PALMPRE,
+    KOBO,
+    AZBOOKA,
+    FOLDER_DEVICE_FOR_CONFIG,
+    AVANT,
+    MENTOR,
+    SWEEX,
+    ITUNES,
 ]
 plugins += [x for x in list(locals().values()) if isinstance(x, type) and \
                                         x.__name__.endswith('MetadataReader')]
