@@ -48,15 +48,6 @@ class EB600(USBMS):
     EBOOK_DIR_CARD_A = ''
     SUPPORTS_SUB_DIRS = True
 
-    def windows_sort_drives(self, drives):
-        main = drives.get('main', None)
-        card = drives.get('carda', None)
-        if card and main and card < main:
-            drives['main'] = card
-            drives['carda'] = main
-
-        return drives
-
 
 class COOL_ER(EB600):
 
@@ -195,3 +186,45 @@ class BOOQ(EB600):
     WINDOWS_MAIN_MEM = 'EB600'
     WINDOWS_CARD_A_MEM = 'EB600'
 
+class MENTOR(EB600):
+
+    name = 'Astak Mentor EB600'
+    gui_name = 'Mentor'
+    description = _('Communicate with the Astak Mentor EB600')
+    FORMATS = ['epub', 'fb2', 'mobi', 'prc', 'pdf', 'txt']
+
+    WINDOWS_MAIN_MEM = WINDOWS_CARD_A_MEM = 'MENTOR'
+
+class ELONEX(EB600):
+
+    name = 'Elonex 600EB'
+    gui_name = 'Elonex'
+
+    FORMATS = ['epub', 'pdf', 'txt', 'html']
+
+    VENDOR_NAME = 'ELONEX'
+    WINDOWS_MAIN_MEM = 'EBOOK'
+    WINDOWS_CARD_A_MEM = 'EBOOK'
+
+    @classmethod
+    def can_handle(cls, dev, debug=False):
+        return dev[3] == 'Elonex' and dev[4] == 'eBook'
+
+class POCKETBOOK301(USBMS):
+
+    name           = 'PocketBook 301 Device Interface'
+    description    = _('Communicate with the PocketBook 301 reader.')
+    author         = 'Kovid Goyal'
+    supported_platforms = ['windows', 'osx', 'linux']
+    FORMATS = ['epub', 'fb2', 'prc', 'mobi', 'pdf', 'djvu', 'rtf', 'chm', 'txt']
+
+    SUPPORTS_SUB_DIRS = True
+
+    MAIN_MEMORY_VOLUME_LABEL  = 'PocketBook 301 Main Memory'
+    STORAGE_CARD_VOLUME_LABEL = 'PocketBook 301 Storage Card'
+
+    VENDOR_ID   = [0x1]
+    PRODUCT_ID  = [0x301]
+    BCD         = [0x132]
+
+
