@@ -6,6 +6,7 @@ the GUI. A device backend must subclass the L{Device} class. See prs500.py for
 a backend that implement the Device interface for the SONY PRS500 Reader.
 """
 import os
+from collections import namedtuple
 
 from calibre.customize import Plugin
 from calibre.constants import iswindows
@@ -43,6 +44,13 @@ class DevicePlugin(Plugin):
     #: Icon for this device
     icon = I('reader.svg')
 
+    # Used by gui2.ui:annotations_fetched() and devices.kindle.driver:get_annotations()
+    UserAnnotation = namedtuple('Annotation','type, value')
+
+    #: GUI displays this as a message if not None. Useful if opening can take a
+    #: long time
+    OPEN_FEEDBACK_MESSAGE = None
+
     @classmethod
     def get_gui_name(cls):
         if hasattr(cls, 'gui_name'):
@@ -51,7 +59,7 @@ class DevicePlugin(Plugin):
             return cls.__name__
         return cls.name
 
-
+    # Device detection {{{
     def test_bcd_windows(self, device_id, bcd):
         if bcd is None or len(bcd) == 0:
             return True
@@ -144,6 +152,7 @@ class DevicePlugin(Plugin):
                                     return True, dev
         return False, None
 
+    # }}}
 
     def reset(self, key='-1', log_packets=False, report_progress=None,
             detected_device=None) :
@@ -279,7 +288,12 @@ class DevicePlugin(Plugin):
         This method should raise a L{FreeSpaceError} if there is not enough
         free space on the device. The text of the FreeSpaceError must contain the
         word "card" if C{on_card} is not None otherwise it must contain the word "memory".
-        :files: A list of paths and/or file-like objects.
+        :files: A list of paths and/or file-like objects. If they are paths and
+        the paths point to temporary files, they may have an additional
+        attribute, original_file_path pointing to the originals. They may have
+        another optional attribute, deleted_after_upload which if True means
+        that the file pointed to by original_file_path will be deleted after
+        being uploaded to the device.
         :names: A list of file names that the books should have
         once uploaded to the device. len(names) == len(files)
         :return: A list of 3-element tuples. The list is meant to be passed
@@ -289,8 +303,7 @@ class DevicePlugin(Plugin):
         put the book. len(metadata) == len(files). Apart from the regular
         cover (path to cover), there may also be a thumbnail attribute, which should
         be used in preference. The thumbnail attribute is of the form
-        (width, height, cover_data as jpeg). In addition the MetaInformation
-        objects can have a tag_order attribute.
+        (width, height, cover_data as jpeg).
         '''
         raise NotImplementedError()
 
@@ -360,14 +373,12 @@ class DevicePlugin(Plugin):
     @classmethod
     def settings(cls):
         '''
-        Should return an opts object. The opts object should have one attribute
+        Should return an opts object. The opts object should have at least one attribute
         `format_map` which is an ordered list of formats for the device.
         '''
         raise NotImplementedError()
 
 
-
-
 class BookList(list):
     '''
     A list of books. Each Book object must have the fields:
@@ -376,22 +387,46 @@ class BookList(list):
       3. size (file size of the book)
       4. datetime (a UTC time tuple)
       5. path (path on the device to the book)
-      6. thumbnail (can be None)
+      6. thumbnail (can be None) thumbnail is either a str/bytes object with the
+         image data or it should have an attribute image_path that stores an
+         absolute (platform native) path to the image
       7. tags (a list of strings, can be empty).
     '''
 
     __getslice__ = None
     __setslice__ = None
 
-    def supports_tags(self):
-        ''' Return True if the the device supports tags (collections) for this book list. '''
+    def __init__(self, oncard, prefix, settings):
+        pass
+
+    def supports_collections(self):
+        ''' Return True if the the device supports collections for this book list. '''
         raise NotImplementedError()
 
-    def set_tags(self, book, tags):
+    def add_book(self, book, replace_metadata):
         '''
-        Set the tags for C{book} to C{tags}.
-        @param tags: A list of strings. Can be empty.
-        @param book: A book object that is in this BookList.
+        Add the book to the booklist. Intent is to maintain any device-internal
+        metadata. Return True if booklists must be sync'ed
+        '''
+        raise NotImplementedError()
+
+    def remove_book(self, book):
+        '''
+        Remove a book from the booklist. Correct any device metadata at the
+        same time
+        '''
+        raise NotImplementedError()
+
+    def get_collections(self, collection_attributes):
+        '''
+        Return a dictionary of collections created from collection_attributes.
+        Each entry in the dictionary is of the form collection name:[list of
+        books]
+
+        The list of books is sorted by book title, except for collections
+        created from series, in which case series_index is used.
+
+        :param collection_attributes: A list of attributes of the Book object
         '''
         raise NotImplementedError()
 
