@@ -11,12 +11,7 @@ intended to be subclassed with the relevant parts implemented for a particular
 device. This class handles device detection.
 '''
 
-import os
-import subprocess
-import time
-import re
-import sys
-import glob
+import os, subprocess, time, re, sys, glob, operator
 from itertools import repeat
 
 from calibre.devices.interface import DevicePlugin
@@ -61,6 +56,8 @@ class Device(DeviceConfig, DevicePlugin):
     BCD         = None
 
     VENDOR_NAME = None
+
+    # These can be None, string, list of strings or compiled regex
     WINDOWS_MAIN_MEM = None
     WINDOWS_CARD_A_MEM = None
     WINDOWS_CARD_B_MEM = None
@@ -75,13 +72,12 @@ class Device(DeviceConfig, DevicePlugin):
     # storage cards. Should be a regular expression that matches the
     # main memory mount point assigned by OS X
     OSX_MAIN_MEM_VOL_PAT = None
+    OSX_EJECT_COMMAND = ['diskutil', 'eject']
 
     MAIN_MEMORY_VOLUME_LABEL  = ''
     STORAGE_CARD_VOLUME_LABEL = ''
     STORAGE_CARD2_VOLUME_LABEL = None
 
-    SUPPORTS_SUB_DIRS = False
-    MUST_READ_METADATA = False
 
     EBOOK_DIR_MAIN = ''
     EBOOK_DIR_CARD_A = ''
@@ -111,15 +107,16 @@ class Device(DeviceConfig, DevicePlugin):
     def _windows_space(cls, prefix):
         if not prefix:
             return 0, 0
+        prefix = prefix[:-1]
         win32file = __import__('win32file', globals(), locals(), [], -1)
         try:
             sectors_per_cluster, bytes_per_sector, free_clusters, total_clusters = \
-                win32file.GetDiskFreeSpace(prefix[:-1])
+                win32file.GetDiskFreeSpace(prefix)
         except Exception, err:
             if getattr(err, 'args', [None])[0] == 21: # Disk not ready
                 time.sleep(3)
                 sectors_per_cluster, bytes_per_sector, free_clusters, total_clusters = \
-                    win32file.GetDiskFreeSpace(prefix[:-1])
+                    win32file.GetDiskFreeSpace(prefix)
             else: raise
         mult = sectors_per_cluster * bytes_per_sector
         return total_clusters * mult, free_clusters * mult
@@ -243,21 +240,26 @@ class Device(DeviceConfig, DevicePlugin):
                 drives.get('main', None) is None:
             drives['main'] = drives.pop('carda')
 
-        drives = self.windows_open_callback(drives)
-
         if drives.get('main', None) is None:
             raise DeviceError(
                 _('Unable to detect the %s disk drive. Try rebooting.') %
                 self.__class__.__name__)
 
+        # Sort drives by their PNP drive numbers if the CARD and MAIN
+        # MEM strings are identical
+        if self.WINDOWS_MAIN_MEM in (self.WINDOWS_CARD_A_MEM,
+                self.WINDOWS_CARD_B_MEM) or \
+                self.WINDOWS_CARD_A_MEM == self.WINDOWS_CARD_B_MEM:
+            letters = sorted(drives.values(), key=operator.attrgetter('order'))
+            drives = {}
+            for which, letter in zip(['main', 'carda', 'cardb'], letters):
+                drives[which] = letter
+
         drives = self.windows_sort_drives(drives)
         self._main_prefix = drives.get('main')
         self._card_a_prefix = drives.get('carda', None)
         self._card_b_prefix = drives.get('cardb', None)
 
-    def windows_open_callback(self, drives):
-        return drives
-
     @classmethod
     def run_ioreg(cls, raw=None):
         if raw is not None:
@@ -333,10 +335,14 @@ class Device(DeviceConfig, DevicePlugin):
                     raise
             time.sleep(2)
 
-    def _osx_bsd_names(self):
+    @classmethod
+    def osx_get_usb_drives(cls):
         if usbobserver_err:
             raise RuntimeError('Failed to load usbobserver: '+usbobserver_err)
-        drives = usbobserver.get_usb_drives()
+        return usbobserver.get_usb_drives()
+
+    def _osx_bsd_names(self):
+        drives = self.osx_get_usb_drives()
         matches = []
         d = self.detected_device
         if d.serial:
@@ -394,16 +400,6 @@ class Device(DeviceConfig, DevicePlugin):
         if len(matches) > 2:
             drives['cardb'] = matches[2]
 
-        pat = self.OSX_MAIN_MEM_VOL_PAT
-        if pat is not None and len(drives) > 1 and 'main' in drives:
-            if pat.search(drives['main']) is None:
-                main = drives['main']
-                for x in ('carda', 'cardb'):
-                    if x in drives and pat.search(drives[x]):
-                        drives['main'] = drives.pop(x)
-                        drives[x] = main
-                        break
-
         return drives
 
     def osx_bsd_names(self):
@@ -427,6 +423,16 @@ class Device(DeviceConfig, DevicePlugin):
         if drives['main'] is None:
             print bsd_drives, mount_map, drives
             raise DeviceError(_('Unable to detect the %s mount point. Try rebooting.')%self.__class__.__name__)
+        pat = self.OSX_MAIN_MEM_VOL_PAT
+        if pat is not None and len(drives) > 1 and 'main' in drives:
+            if pat.search(drives['main']) is None:
+                main = drives['main']
+                for x in ('carda', 'cardb'):
+                    if x in drives and pat.search(drives[x]):
+                        drives['main'] = drives.pop(x)
+                        drives[x] = main
+                        break
+
         self._main_prefix = drives['main']+os.sep
         def get_card_prefix(c):
             ans = drives.get(c, None)
@@ -664,7 +670,7 @@ class Device(DeviceConfig, DevicePlugin):
             x = getattr(self, x, None)
             if x is not None:
                 try:
-                    subprocess.Popen(['diskutil', 'eject', x])
+                    subprocess.Popen(self.OSX_EJECT_COMMAND + [x])
                 except:
                     pass
 
@@ -757,12 +763,8 @@ class Device(DeviceConfig, DevicePlugin):
             path = existing[0]
 
         def get_size(obj):
-            if hasattr(obj, 'seek'):
-                obj.seek(0, os.SEEK_END)
-                size = obj.tell()
-                obj.seek(0)
-                return size
-            return os.path.getsize(obj)
+            path = getattr(obj, 'name', obj)
+            return os.path.getsize(path)
 
         sizes = [get_size(f) for f in files]
         size = sum(sizes)
@@ -789,7 +791,13 @@ class Device(DeviceConfig, DevicePlugin):
         '''
         return components
 
-    def create_upload_path(self, path, mdata, fname):
+    def get_annotations(self, path_map):
+        '''
+        Resolve path_map to annotation_map of files found on the device
+        '''
+        return {}
+
+    def create_upload_path(self, path, mdata, fname, create_dirs=True):
         path = os.path.abspath(path)
         extra_components = []
 
@@ -802,6 +810,9 @@ class Device(DeviceConfig, DevicePlugin):
 
         settings = self.settings()
         template = self.save_template()
+        if mdata.tags and _('News') in mdata.tags:
+            today = time.localtime()
+            template = "{title}_%d-%d-%d" % (today[0], today[1], today[2])
         use_subdirs = self.SUPPORTS_SUB_DIRS and settings.use_subdirs
 
         fname = sanitize(fname)
@@ -810,13 +821,19 @@ class Device(DeviceConfig, DevicePlugin):
         from calibre.library.save_to_disk import get_components
         if not isinstance(template, unicode):
             template = template.decode('utf-8')
-        extra_components = get_components(template, mdata, fname)
+        app_id = str(getattr(mdata, 'application_id', ''))
+        # The db id will be in the created filename
+        extra_components = get_components(template, mdata, fname,
+                length=250-len(app_id)-1)
         if not extra_components:
             extra_components.append(sanitize(self.filename_callback(fname,
                 mdata)))
         else:
             extra_components[-1] = sanitize(self.filename_callback(extra_components[-1]+ext, mdata))
 
+        if extra_components[-1] and extra_components[-1][0] in ('.', '_'):
+            extra_components[-1] = 'x' + extra_components[-1][1:]
+
         if special_tag is not None:
             name = extra_components[-1]
             extra_components = []
@@ -848,7 +865,7 @@ class Device(DeviceConfig, DevicePlugin):
         filedir = os.path.dirname(filepath)
 
 
-        if not os.path.exists(filedir):
+        if create_dirs and not os.path.exists(filedir):
             os.makedirs(filedir)
 
         return filepath
