@@ -171,7 +171,8 @@ class PageProcessor(list):
                 if not self.opts.dont_sharpen:
                     p.MagickSharpenImage(wand, 0.0, 1.0)
 
-                p.MagickSetImageType(wand, p.GrayscaleType)
+                if not self.opts.dont_grayscale:
+                    p.MagickSetImageType(wand, p.GrayscaleType)
 
                 if self.opts.despeckle:
                     p.MagickDespeckleImage(wand)
@@ -256,7 +257,7 @@ def process_pages(pages, opts, update, tdir):
     ans, failures = [], []
 
     for job in jobs:
-        if job.failed:
+        if job.failed or job.result is None:
             raise Exception(_('Failed to process comic: \n\n%s')%
                     job.log_file.read())
         pages, failures_ = job.result
@@ -310,6 +311,8 @@ class ComicInput(InputFormatPlugin):
                 'you optimal size and look on your device.')),
         OptionRecommendation(name='no_process', recommended_value=False,
               help=_("Apply no processing to the image")),
+        OptionRecommendation(name='dont_grayscale', recommended_value=False,
+            help=_('Do not convert the image to grayscale (black and white)'))
         ])
 
     recommendations = set([
@@ -319,7 +322,7 @@ class ComicInput(InputFormatPlugin):
         ('margin_bottom', 0, OptionRecommendation.HIGH),
         ('insert_blank_line', False, OptionRecommendation.HIGH),
         ('remove_paragraph_spacing',  False, OptionRecommendation.HIGH),
-        ('dont_justify', True, OptionRecommendation.HIGH),
+        ('change_justification', 'left', OptionRecommendation.HIGH),
         ('dont_split_on_pagebreaks', True, OptionRecommendation.HIGH),
         ('chapter', None, OptionRecommendation.HIGH),
         ('page_breaks_brefore', None, OptionRecommendation.HIGH),
@@ -338,8 +341,15 @@ class ComicInput(InputFormatPlugin):
             if not os.path.exists('comics.txt'):
                 raise ValueError('%s is not a valid comic collection'
                         %stream.name)
-            raw = open('comics.txt', 'rb').read().decode('utf-8')
-            raw.lstrip(unicode(codecs.BOM_UTF8, "utf8" ))
+            raw = open('comics.txt', 'rb').read()
+            if raw.startswith(codecs.BOM_UTF16_BE):
+                raw = raw.decode('utf-16-be')[1:]
+            elif raw.startswith(codecs.BOM_UTF16_LE):
+                raw = raw.decode('utf-16-le')[1:]
+            elif raw.startswith(codecs.BOM_UTF8):
+                raw = raw.decode('utf-8')[1:]
+            else:
+                raw = raw.decode('utf-8')
             for line in raw.splitlines():
                 line = line.strip()
                 if not line:
