@@ -18,7 +18,7 @@ from calibre.constants import __appname__, __version__, filesystem_encoding
 from calibre.ebooks.metadata.toc import TOC
 from calibre.ebooks.metadata import MetaInformation, string_to_authors
 from calibre.utils.date import parse_date, isoformat
-
+from calibre.utils.localization import get_lang
 
 class Resource(object):
     '''
@@ -404,6 +404,10 @@ class MetadataField(object):
 
     def __set__(self, obj, val):
         elem = obj.get_metadata_element(self.name)
+        if val is None:
+            if elem is not None:
+                elem.getparent().remove(elem)
+            return
         if elem is None:
             elem = obj.create_metadata_element(self.name, is_dc=self.is_dc)
         obj.set_text(elem, unicode(val))
@@ -433,6 +437,8 @@ class OPF(object):
     tags_path       = XPath('descendant::*[re:match(name(), "subject", "i")]')
     isbn_path       = XPath('descendant::*[re:match(name(), "identifier", "i") and '+
                             '(re:match(@scheme, "isbn", "i") or re:match(@opf:scheme, "isbn", "i"))]')
+    raster_cover_path = XPath('descendant::*[re:match(name(), "meta", "i") and ' +
+            're:match(@name, "cover", "i") and @content]')
     identifier_path = XPath('descendant::*[re:match(name(), "identifier", "i")]')
     application_id_path = XPath('descendant::*[re:match(name(), "identifier", "i") and '+
                             '(re:match(@opf:scheme, "calibre|libprs500", "i") or re:match(@scheme, "calibre|libprs500", "i"))]')
@@ -445,8 +451,6 @@ class OPF(object):
     guide_path      = XPath('descendant::*[re:match(name(), "guide", "i")]/*[re:match(name(), "reference", "i")]')
 
     title           = MetadataField('title', formatter=lambda x: re.sub(r'\s+', ' ', x))
-    title_sort      = MetadataField('title_sort', formatter=lambda x:
-                        re.sub(r'\s+', ' ', x), is_dc=False)
     publisher       = MetadataField('publisher')
     language        = MetadataField('language')
     comments        = MetadataField('description')
@@ -470,7 +474,8 @@ class OPF(object):
             raise ValueError('Empty file: '+getattr(stream, 'name', 'stream'))
         self.basedir  = self.base_dir = basedir
         self.path_to_html_toc = self.html_toc_fragment = None
-        raw, self.encoding = xml_to_unicode(raw, strip_encoding_pats=True, resolve_entities=True)
+        raw, self.encoding = xml_to_unicode(raw, strip_encoding_pats=True,
+                resolve_entities=True, assume_utf8=True)
         raw = raw[raw.find('<'):]
         self.root     = etree.fromstring(raw, self.PARSER)
         self.metadata = self.metadata_path(self.root)
@@ -662,10 +667,19 @@ class OPF(object):
             remove = list(self.authors_path(self.metadata))
             for elem in remove:
                 elem.getparent().remove(elem)
+            elems = []
             for author in val:
                 attrib = {'{%s}role'%self.NAMESPACES['opf']: 'aut'}
                 elem = self.create_metadata_element('creator', attrib=attrib)
                 self.set_text(elem, author.strip())
+                # Ensure new author element is at the top of the list
+                # for broken implementations that always use the first
+                # <dc:creator> element with no attention to the role
+                elems.append(elem)
+            for elem in reversed(elems):
+                parent = elem.getparent()
+                parent.remove(elem)
+                parent.insert(0, elem)
 
         return property(fget=fget, fset=fset)
 
@@ -692,18 +706,44 @@ class OPF(object):
 
         return property(fget=fget, fset=fset)
 
+    @dynamic_property
+    def title_sort(self):
+
+        def fget(self):
+            matches = self.title_path(self.metadata)
+            if matches:
+                for match in matches:
+                    ans = match.get('{%s}file-as'%self.NAMESPACES['opf'], None)
+                    if not ans:
+                        ans = match.get('file-as', None)
+                    if ans:
+                        return ans
+
+        def fset(self, val):
+            matches = self.title_path(self.metadata)
+            if matches:
+                for key in matches[0].attrib:
+                    if key.endswith('file-as'):
+                        matches[0].attrib.pop(key)
+                matches[0].set('{%s}file-as'%self.NAMESPACES['opf'], unicode(val))
+
+        return property(fget=fget, fset=fset)
+
+
     @dynamic_property
     def tags(self):
 
         def fget(self):
             ans = []
             for tag in self.tags_path(self.metadata):
-                ans.append(self.get_text(tag))
+                text = self.get_text(tag)
+                if text and text.strip():
+                    ans.extend([x.strip() for x in text.split(',')])
             return ans
 
         def fset(self, val):
             for tag in list(self.tags_path(self.metadata)):
-                self.metadata.remove(tag)
+                tag.getparent().remove(tag)
             for tag in val:
                 elem = self.create_metadata_element('subject')
                 self.set_text(elem, unicode(tag))
@@ -719,6 +759,11 @@ class OPF(object):
 
         def fset(self, val):
             matches = self.isbn_path(self.metadata)
+            if val is None:
+                if matches:
+                    for x in matches:
+                        x.getparent().remove(x)
+                    return
             if not matches:
                 attrib = {'{%s}scheme'%self.NAMESPACES['opf']: 'ISBN'}
                 matches = [self.create_metadata_element('identifier',
@@ -779,6 +824,9 @@ class OPF(object):
             self.set_text(matches[0], unicode(val))
         return property(fget=fget, fset=fset)
 
+    def identifier_iter(self):
+        for item in self.identifier_path(self.metadata):
+            yield item
 
     def guess_cover(self):
         '''
@@ -800,6 +848,14 @@ class OPF(object):
                         if os.access(os.path.join(self.base_dir, prefix+suffix), os.R_OK):
                             return cpath
 
+    @property
+    def raster_cover(self):
+        covers = self.raster_cover_path(self.metadata)
+        if covers:
+            cover_id = covers[0].get('content')
+            for item in self.itermanifest():
+                if item.get('id', None) == cover_id:
+                    return item.get('href', None)
 
     @dynamic_property
     def cover(self):
@@ -949,11 +1005,8 @@ class OPFCreator(MetaInformation):
 
     def render(self, opf_stream=sys.stdout, ncx_stream=None,
                ncx_manifest_entry=None, encoding=None):
-        from calibre.utils.genshi.template import MarkupTemplate
-        opf_template = open(P('templates/opf.xml'), 'rb').read()
         if encoding is None:
             encoding = 'utf-8'
-        template = MarkupTemplate(opf_template)
         toc = getattr(self, 'toc', None)
         if self.manifest:
             self.manifest.set_basedir(self.base_path)
@@ -974,12 +1027,104 @@ class OPFCreator(MetaInformation):
                 cover = os.path.abspath(os.path.join(self.base_path, cover))
             self.guide.set_cover(cover)
         self.guide.set_basedir(self.base_path)
-        opf = template.generate(
-                __appname__=__appname__, mi=self,
-                __version__=__version__).render('xml', encoding=encoding)
-        opf_stream.write('<?xml version="1.0" encoding="%s" ?>\n'
-                %encoding.upper())
-        opf_stream.write(opf)
+
+        # Actual rendering
+        from lxml.builder import ElementMaker
+        from calibre.ebooks.oeb.base import OPF2_NS, DC11_NS, CALIBRE_NS
+        DNS = OPF2_NS+'___xx___'
+        E = ElementMaker(namespace=DNS, nsmap={None:DNS})
+        M = ElementMaker(namespace=DNS,
+                nsmap={'dc':DC11_NS, 'calibre':CALIBRE_NS, 'opf':OPF2_NS})
+        DC = ElementMaker(namespace=DC11_NS)
+
+        def DC_ELEM(tag, text, dc_attrs={}, opf_attrs={}):
+            if text:
+                elem = getattr(DC, tag)(text, **dc_attrs)
+            else:
+                elem = getattr(DC, tag)(**dc_attrs)
+            for k, v in opf_attrs.items():
+                elem.set('{%s}%s'%(OPF2_NS, k), v)
+            return elem
+
+        def CAL_ELEM(name, content):
+            return M.meta(name=name, content=content)
+
+        metadata = M.metadata()
+        a = metadata.append
+        role = {}
+        if self.title_sort:
+            role = {'file-as':self.title_sort}
+        a(DC_ELEM('title', self.title if self.title else _('Unknown'),
+            opf_attrs=role))
+        for i, author in enumerate(self.authors):
+            fa = {'role':'aut'}
+            if i == 0 and self.author_sort:
+                fa['file-as'] = self.author_sort
+            a(DC_ELEM('creator', author, opf_attrs=fa))
+        a(DC_ELEM('contributor', '%s (%s) [%s]'%(__appname__, __version__,
+            'http://calibre-ebook.com'), opf_attrs={'role':'bkp',
+                'file-as':__appname__}))
+        a(DC_ELEM('identifier', str(self.application_id),
+            opf_attrs={'scheme':__appname__},
+            dc_attrs={'id':__appname__+'_id'}))
+        if getattr(self, 'pubdate', None) is not None:
+            a(DC_ELEM('date', self.pubdate.isoformat()))
+        lang = self.language
+        if not lang or lang.lower() == 'und':
+            lang = get_lang().replace('_', '-')
+        a(DC_ELEM('language', lang))
+        if self.comments:
+            a(DC_ELEM('description', self.comments))
+        if self.publisher:
+            a(DC_ELEM('publisher', self.publisher))
+        if self.isbn:
+            a(DC_ELEM('identifier', self.isbn, opf_attrs={'scheme':'ISBN'}))
+        if self.rights:
+            a(DC_ELEM('rights', self.rights))
+        if self.tags:
+            for tag in self.tags:
+                a(DC_ELEM('subject', tag))
+        if self.series:
+            a(CAL_ELEM('calibre:series', self.series))
+            if self.series_index is not None:
+                a(CAL_ELEM('calibre:series_index', self.format_series_index()))
+        if self.rating is not None:
+            a(CAL_ELEM('calibre:rating', str(self.rating)))
+        if self.timestamp is not None:
+            a(CAL_ELEM('calibre:timestamp', self.timestamp.isoformat()))
+        if self.publication_type is not None:
+            a(CAL_ELEM('calibre:publication_type', self.publication_type))
+        manifest = E.manifest()
+        if self.manifest is not None:
+            for ref in self.manifest:
+                item = E.item(id=str(ref.id), href=ref.href())
+                item.set('media-type', ref.mime_type)
+                manifest.append(item)
+        spine = E.spine()
+        if self.toc is not None:
+            spine.set('toc', 'ncx')
+        if self.spine is not None:
+            for ref in self.spine:
+                spine.append(E.itemref(idref=ref.id))
+        guide = E.guide()
+        if self.guide is not None:
+            for ref in self.guide:
+                item = E.reference(type=ref.type, href=ref.href())
+                if ref.title:
+                    item.set('title', ref.title)
+                guide.append(item)
+
+        root = E.package(
+                metadata,
+                manifest,
+                spine,
+                guide
+        )
+        root.set('unique-identifier', __appname__+'_id')
+        raw = etree.tostring(root, pretty_print=True, xml_declaration=True,
+                encoding=encoding)
+        raw = raw.replace(DNS, OPF2_NS)
+        opf_stream.write(raw)
         opf_stream.flush()
         if toc is not None and ncx_stream is not None:
             toc.render(ncx_stream, self.application_id)
@@ -1042,7 +1187,6 @@ def metadata_to_opf(mi, as_string=True):
     factory(DC('contributor'), mi.book_producer, __appname__, 'bkp')
     if hasattr(mi.pubdate, 'isoformat'):
         factory(DC('date'), isoformat(mi.pubdate))
-    factory(DC('language'), mi.language)
     if mi.category:
         factory(DC('type'), mi.category)
     if mi.comments:
@@ -1053,6 +1197,8 @@ def metadata_to_opf(mi, as_string=True):
         factory(DC('identifier'), mi.isbn, scheme='ISBN')
     if mi.rights:
         factory(DC('rights'), mi.rights)
+    factory(DC('language'), mi.language if mi.language and mi.language.lower()
+            != 'und' else get_lang().replace('_', '-'))
     if mi.tags:
         for tag in mi.tags:
             factory(DC('subject'), tag)
@@ -1127,12 +1273,14 @@ class OPFTest(unittest.TestCase):
 <package version="2.0" xmlns="http://www.idpf.org/2007/opf" >
 <metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
     <dc:title opf:file-as="Wow">A Cool &amp; &copy; &#223; Title</dc:title>
-    <creator opf:role="aut" file-as="Monkey">Monkey Kitchen, Next</creator>
+    <creator opf:role="aut" file-as="Monkey">Monkey Kitchen</creator>
+    <creator opf:role="aut">Next</creator>
     <dc:subject>One</dc:subject><dc:subject>Two</dc:subject>
     <dc:identifier scheme="ISBN">123456789</dc:identifier>
-    <x-metadata>
-        <series>A one book series</series>
-    </x-metadata>
+    <meta name="calibre:series" content="A one book series" />
+    <meta name="calibre:rating" content="4"/>
+    <meta name="calibre:publication_type" content="test"/>
+    <meta name="calibre:series_index" content="2.5" />
 </metadata>
 <manifest>
     <item id="1" href="a%20%7E%20b" media-type="text/txt" />
@@ -1152,7 +1300,9 @@ class OPFTest(unittest.TestCase):
         self.assertEqual(opf.tags, ['One', 'Two'])
         self.assertEqual(opf.isbn, '123456789')
         self.assertEqual(opf.series, 'A one book series')
-        self.assertEqual(opf.series_index, 1)
+        self.assertEqual(opf.series_index, 2.5)
+        self.assertEqual(opf.rating, 4)
+        self.assertEqual(opf.publication_type, 'test')
         self.assertEqual(list(opf.itermanifest())[0].get('href'), 'a ~ b')
 
     def testWriting(self):
@@ -1182,3 +1332,6 @@ def suite():
 
 def test():
     unittest.TextTestRunner(verbosity=2).run(suite())
+
+if __name__ == '__main__':
+    test()
