@@ -3,9 +3,10 @@ __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import os
-from zipfile import ZipFile
-from cStringIO import StringIO
 
+from calibre.utils.zipfile import ZipFile
+from calibre.ptempfile import TemporaryDirectory
+from calibre import CurrentDir
 
 def get_metadata(stream):
     from calibre.ebooks.metadata.meta import get_metadata
@@ -22,9 +23,11 @@ def get_metadata(stream):
         if stream_type:
             stream_type = stream_type[1:]
             if stream_type in ('lit', 'opf', 'prc', 'mobi', 'fb2', 'epub',
-                               'rb', 'imp', 'pdf', 'lrf'):
-                stream = StringIO(zf.read(f))
-                return get_metadata(stream, stream_type)
+                               'rb', 'imp', 'pdf', 'lrf', 'azw'):
+                with TemporaryDirectory() as tdir:
+                    with CurrentDir(tdir):
+                        path = zf.extract(f)
+                        return get_metadata(open(path, 'rb'), stream_type)
     raise ValueError('No ebook found in ZIP archive')
 
 
