@@ -4,12 +4,7 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 Read data from .mobi files
 '''
 
-import functools
-import os
-import re
-import struct
-import textwrap
-import cStringIO
+import shutil, os, re, struct, textwrap, cStringIO, sys
 
 try:
     from PIL import Image as PILImage
@@ -19,7 +14,7 @@ except ImportError:
 
 from lxml import html, etree
 
-from calibre import entity_to_unicode, CurrentDir
+from calibre import xml_entity_to_unicode, CurrentDir, entity_to_unicode
 from calibre.utils.filenames import ascii_filename
 from calibre.utils.date import parse_date
 from calibre.ptempfile import TemporaryDirectory
@@ -66,7 +61,8 @@ class EXTHHeader(object):
                 # last update time
                 pass
             elif id == 503: # Long title
-                if not title or title == _('Unknown'):
+                if not title or title == _('Unknown') or \
+                        'USER_CONTENT' in title or title.startswith('dtp_'):
                     try:
                         title = content.decode(codec)
                     except:
@@ -258,6 +254,8 @@ class MobiReader(object):
             stream = open(filename_or_stream, 'rb')
 
         raw = stream.read()
+        if raw.startswith('TPZ'):
+            raise ValueError(_('This is an Amazon Topaz book. It cannot be processed.'))
 
         self.header   = raw[0:72]
         self.name     = self.header[:32].replace('\x00', '')
@@ -265,7 +263,7 @@ class MobiReader(object):
 
         self.ident = self.header[0x3C:0x3C + 8].upper()
         if self.ident not in ['BOOKMOBI', 'TEXTREAD']:
-            raise MobiError('Unknown book type: %s' % self.ident)
+            raise MobiError('Unknown book type: %s' % repr(self.ident))
 
         self.sections = []
         self.section_headers = []
@@ -301,11 +299,13 @@ class MobiReader(object):
         self.add_anchors()
         self.processed_html = self.processed_html.decode(self.book_header.codec,
             'ignore')
+        self.processed_html = self.processed_html.replace('</</', '</')
+        self.processed_html = re.sub(r'</([a-zA-Z]+)<', r'</\1><',
+                self.processed_html)
+
         for pat in ENCODING_PATS:
             self.processed_html = pat.sub('', self.processed_html)
-        e2u = functools.partial(entity_to_unicode,
-            exceptions=['lt', 'gt', 'amp', 'apos', 'quot', '#60', '#62'])
-        self.processed_html = re.sub(r'&(\S+?);', e2u,
+        self.processed_html = re.sub(r'&(\S+?);', xml_entity_to_unicode,
             self.processed_html)
         self.extract_images(processed_records, output_dir)
         self.replace_page_breaks()
@@ -500,8 +500,8 @@ class MobiReader(object):
                 if ':' in x:
                     del tag.attrib[x]
             if tag.tag in ('country-region', 'place', 'placetype', 'placename',
-                'state', 'city', 'street', 'address', 'content'):
-                tag.tag = 'div' if tag.tag == 'content' else 'span'
+                'state', 'city', 'street', 'address', 'content', 'form'):
+                tag.tag = 'div' if tag.tag in ('content', 'form') else 'span'
                 for key in tag.attrib.keys():
                     tag.attrib.pop(key)
                 continue
@@ -619,11 +619,28 @@ class MobiReader(object):
                 * opf.cover.split('/'))):
                 opf.cover = None
 
+        cover = opf.cover
+        cover_copied = None
+        if cover is not None:
+            cover = cover.replace('/', os.sep)
+            if os.path.exists(cover):
+                ncover = 'images'+os.sep+'calibre_cover.jpg'
+                if os.path.exists(ncover):
+                    os.remove(ncover)
+                shutil.copyfile(cover, ncover)
+                cover_copied = os.path.abspath(ncover)
+                opf.cover = ncover.replace(os.sep, '/')
+
         manifest = [(htmlfile, 'application/xhtml+xml'),
             (os.path.abspath('styles.css'), 'text/css')]
         bp = os.path.dirname(htmlfile)
+        added = set([])
         for i in getattr(self, 'image_names', []):
-            manifest.append((os.path.join(bp, 'images/', i), 'image/jpeg'))
+            path = os.path.join(bp, 'images', i)
+            added.add(path)
+            manifest.append((path, 'image/jpeg'))
+        if cover_copied is not None:
+            manifest.append((cover_copied, 'image/jpeg'))
 
         opf.create_manifest(manifest)
         opf.create_spine([os.path.basename(htmlfile)])
@@ -796,15 +813,22 @@ class MobiReader(object):
 def get_metadata(stream):
     from calibre.utils.logging import Log
     log = Log()
-
     mi = MetaInformation(os.path.basename(stream.name), [_('Unknown')])
-    try:
-        mh = MetadataHeader(stream, log)
+    mh = MetadataHeader(stream, log)
+    if mh.title and mh.title != _('Unknown'):
+        mi.title = mh.title
 
-        if mh.exth is not None:
-            if mh.exth.mi is not None:
-                mi = mh.exth.mi
-        else:
+    if mh.exth is not None:
+        if mh.exth.mi is not None:
+            mi = mh.exth.mi
+    else:
+        size = sys.maxint
+        if hasattr(stream, 'seek') and hasattr(stream, 'tell'):
+            pos = stream.tell()
+            stream.seek(0, 2)
+            size = stream.tell()
+            stream.seek(pos)
+        if size < 4*1024*1024:
             with TemporaryDirectory('_mobi_meta_reader') as tdir:
                 with CurrentDir(tdir):
                     mr = MobiReader(stream, log)
@@ -812,16 +836,18 @@ def get_metadata(stream):
                     mr.extract_content(tdir, parse_cache)
                     if mr.embedded_mi is not None:
                         mi = mr.embedded_mi
-        if hasattr(mh.exth, 'cover_offset'):
-            cover_index = mh.first_image_index + mh.exth.cover_offset
-            data  = mh.section_data(int(cover_index))
-        else:
-            data  = mh.section_data(mh.first_image_index)
-        buf = cStringIO.StringIO(data)
+    if hasattr(mh.exth, 'cover_offset'):
+        cover_index = mh.first_image_index + mh.exth.cover_offset
+        data  = mh.section_data(int(cover_index))
+    else:
+        data  = mh.section_data(mh.first_image_index)
+    buf = cStringIO.StringIO(data)
+    try:
         im = PILImage.open(buf)
-        obuf = cStringIO.StringIO()
-        im.convert('RGBA').save(obuf, format='JPEG')
-        mi.cover_data = ('jpg', obuf.getvalue())
     except:
-        log.exception()
+        log.exception('Failed to read MOBI cover')
+    else:
+        obuf = cStringIO.StringIO()
+        im.convert('RGB').save(obuf, format='JPEG')
+        mi.cover_data = ('jpg', obuf.getvalue())
     return mi
