@@ -17,6 +17,7 @@ from urlparse import urljoin
 
 from lxml import etree, html
 from cssutils import CSSParser
+from cssutils.css import CSSRule
 
 import calibre
 from calibre.constants import filesystem_encoding
@@ -25,6 +26,8 @@ from calibre.ebooks.chardet import xml_to_unicode
 from calibre.ebooks.oeb.entitydefs import ENTITYDEFS
 from calibre.ebooks.conversion.preprocess import CSSPreProcessor
 
+RECOVER_PARSER = etree.XMLParser(recover=True, no_network=True, huge_tree=True)
+
 XML_NS       = 'http://www.w3.org/XML/1998/namespace'
 XHTML_NS     = 'http://www.w3.org/1999/xhtml'
 OEB_DOC_NS   = 'http://openebook.org/namespaces/oeb-document/1.0/'
@@ -232,8 +235,6 @@ PREFIXNAME_RE = re.compile(r'^[^:]+[:][^:]+')
 XMLDECL_RE    = re.compile(r'^\s*<[?]xml.*?[?]>')
 CSSURL_RE     = re.compile(r'''url[(](?P<q>["']?)(?P<url>[^)]+)(?P=q)[)]''')
 
-RECOVER_PARSER = etree.XMLParser(recover=True)
-
 
 def element(parent, *args, **kwargs):
     if parent is not None:
@@ -294,6 +295,9 @@ def xml2str(root, pretty_print=False, strip_comments=False):
 def xml2unicode(root, pretty_print=False):
     return etree.tostring(root, pretty_print=pretty_print)
 
+def xml2text(elem):
+    return etree.tostring(elem, method='text', encoding=unicode, with_tail=False)
+
 ASCII_CHARS   = set(chr(x) for x in xrange(128))
 UNIBYTE_CHARS = set(chr(x) for x in xrange(256))
 URL_SAFE      = set('ABCDEFGHIJKLMNOPQRSTUVWXYZ'
@@ -430,7 +434,10 @@ class DirContainer(object):
             return f.write(data)
 
     def exists(self, path):
-        path = os.path.join(self.rootdir, urlunquote(path))
+        try:
+            path = os.path.join(self.rootdir, urlunquote(path))
+        except ValueError: #Happens if path contains quoted special chars
+            return False
         return os.path.isfile(path)
 
     def namelist(self):
@@ -756,6 +763,7 @@ class Manifest(object):
             self.href = self.path = urlnormalize(href)
             self.media_type = media_type
             self.fallback = fallback
+            self.override_css_fetch = None
             self.spine_position = None
             self.linear = True
             if loader is None and data is None:
@@ -768,18 +776,11 @@ class Manifest(object):
                 % (self.id, self.href, self.media_type)
 
         def _parse_xml(self, data):
-            data = xml_to_unicode(data, strip_encoding_pats=True)[0]
+            data = xml_to_unicode(data, strip_encoding_pats=True,
+                    assume_utf8=True, resolve_entities=True)[0]
             if not data:
                 return None
-            parser = etree.XMLParser(recover=True)
-            try:
-                return etree.fromstring(data, parser=parser)
-            except etree.XMLSyntaxError, err:
-                if getattr(err, 'code', 0) == 26 or str(err).startswith('Entity'):
-                    data = xml_to_unicode(data, strip_encoding_pats=True,
-                            resolve_entities=True)[0]
-                    return etree.fromstring(data)
-                raise
+            return etree.fromstring(data, parser=RECOVER_PARSER)
 
         def _parse_xhtml(self, data):
             self.oeb.log.debug('Parsing', self.href, '...')
@@ -787,7 +788,6 @@ class Manifest(object):
             data = self.oeb.decode(data)
             data = self.oeb.html_preprocessor(data)
 
-
             # Remove DOCTYPE declaration as it messes up parsing
             # In particular, it causes tostring to insert xmlns
             # declarations, which messes up the coercing logic
@@ -808,16 +808,17 @@ class Manifest(object):
                         pat = re.compile(r'&(%s);'%('|'.join(user_entities.keys())))
                         data = pat.sub(lambda m:user_entities[m.group(1)], data)
 
+            parser = etree.XMLParser(no_network=True, huge_tree=True)
             # Try with more & more drastic measures to parse
             def first_pass(data):
                 try:
-                    data = etree.fromstring(data)
+                    data = etree.fromstring(data, parser=parser)
                 except etree.XMLSyntaxError, err:
                     self.oeb.log.exception('Initial parse failed:')
                     repl = lambda m: ENTITYDEFS.get(m.group(1), m.group(0))
                     data = ENTITY_RE.sub(repl, data)
                     try:
-                        data = etree.fromstring(data)
+                        data = etree.fromstring(data, parser=parser)
                     except etree.XMLSyntaxError, err:
                         self.oeb.logger.warn('Parsing file %r as HTML' % self.href)
                         if err.args and err.args[0].startswith('Excessive depth'):
@@ -831,7 +832,7 @@ class Manifest(object):
                                 elem.text = elem.text.strip('-')
                         data = etree.tostring(data, encoding=unicode)
                         try:
-                            data = etree.fromstring(data)
+                            data = etree.fromstring(data, parser=parser)
                         except etree.XMLSyntaxError:
                             data = etree.fromstring(data, parser=RECOVER_PARSER)
                 return data
@@ -843,7 +844,7 @@ class Manifest(object):
                 nroot = etree.fromstring('<html></html>')
                 has_body = False
                 for child in list(data):
-                    if barename(child.tag) == 'body':
+                    if isinstance(child.tag, (unicode, str)) and barename(child.tag) == 'body':
                         has_body = True
                         break
                 parent = nroot
@@ -865,12 +866,12 @@ class Manifest(object):
                 data = etree.tostring(data, encoding=unicode)
 
                 try:
-                    data = etree.fromstring(data)
+                    data = etree.fromstring(data, parser=parser)
                 except:
                     data = data.replace(':=', '=').replace(':>', '>')
                     data = data.replace('<http:/>', '')
                     try:
-                        data = etree.fromstring(data)
+                        data = etree.fromstring(data, parser=parser)
                     except etree.XMLSyntaxError:
                         self.oeb.logger.warn('Stripping comments and meta tags from %s'%
                                 self.href)
@@ -881,7 +882,7 @@ class Manifest(object):
                                 "<?xml version='1.0' encoding='utf-8'?><o:p></o:p>",
                                 '')
                         data = data.replace("<?xml version='1.0' encoding='utf-8'??>", '')
-                        data = etree.fromstring(data)
+                        data = etree.fromstring(data, parser=RECOVER_PARSER)
             elif namespace(data.tag) != XHTML_NS:
                 # OEB_DOC_NS, but possibly others
                 ns = namespace(data.tag)
@@ -983,15 +984,40 @@ class Manifest(object):
 
 
         def _parse_css(self, data):
+
+            def get_style_rules_from_import(import_rule):
+                ans = []
+                if not import_rule.styleSheet:
+                    return ans
+                rules = import_rule.styleSheet.cssRules
+                for rule in rules:
+                    if rule.type == CSSRule.IMPORT_RULE:
+                        ans.extend(get_style_rules_from_import(rule))
+                    elif rule.type in (CSSRule.FONT_FACE_RULE,
+                            CSSRule.STYLE_RULE):
+                        ans.append(rule)
+                return ans
+
             self.oeb.log.debug('Parsing', self.href, '...')
             data = self.oeb.decode(data)
-            data = self.oeb.css_preprocessor(data)
-            data = XHTML_CSS_NAMESPACE + data
+            data = self.oeb.css_preprocessor(data, add_namespace=True)
             parser = CSSParser(loglevel=logging.WARNING,
-                               fetcher=self._fetch_css,
+                               fetcher=self.override_css_fetch or self._fetch_css,
                                log=_css_logger)
             data = parser.parseString(data, href=self.href)
             data.namespaces['h'] = XHTML_NS
+            import_rules = list(data.cssRules.rulesOfType(CSSRule.IMPORT_RULE))
+            rules_to_append = []
+            insert_index = None
+            for r in data.cssRules.rulesOfType(CSSRule.STYLE_RULE):
+                insert_index = data.cssRules.index(r)
+                break
+            for rule in import_rules:
+                rules_to_append.extend(get_style_rules_from_import(rule))
+            for r in reversed(rules_to_append):
+                data.insertRule(r, index=insert_index)
+            for rule in import_rules:
+                data.deleteRule(rule)
             return data
 
         def _fetch_css(self, path):
@@ -1048,6 +1074,29 @@ class Manifest(object):
                 self._data = None
             return property(fget, fset, fdel, doc=doc)
 
+        def unload_data_from_memory(self, memory=None):
+            if isinstance(self._data, (str, bytes)):
+                if memory is None:
+                    from calibre.ptempfile import PersistentTemporaryFile
+                    pt = PersistentTemporaryFile(suffix='_oeb_base_mem_unloader.img')
+                    pt.write(self._data)
+                    pt.close()
+                    def loader(*args):
+                        with open(pt.name, 'rb') as f:
+                            ans = f.read()
+                        os.remove(pt.name)
+                        return ans
+                    self._loader = loader
+                else:
+                    def loader2(*args):
+                        with open(memory, 'rb') as f:
+                            ans = f.read()
+                        return ans
+                    self._loader = loader2
+                self._data = None
+
+
+
         def __str__(self):
             data = self.data
             if isinstance(data, etree._Element):
@@ -1167,7 +1216,8 @@ class Manifest(object):
         if item in self.ids:
             item = self.ids[item]
         del self.ids[item.id]
-        del self.hrefs[item.href]
+        if item.href in self.hrefs:
+            del self.hrefs[item.href]
         self.items.remove(item)
         if item in self.oeb.spine:
             self.oeb.spine.remove(item)
@@ -1191,7 +1241,8 @@ class Manifest(object):
             href = urlnormalize(href)
             base, ext = os.path.splitext(href)
             index = 1
-            while href in self.hrefs:
+            lhrefs = set([x.lower() for x in self.hrefs])
+            while href.lower() in lhrefs:
                 href = base + str(index) + ext
                 index += 1
         return id, href
