@@ -18,10 +18,12 @@ from calibre.ebooks.chardet import xml_to_unicode
 from calibre.utils.zipfile import safe_replace, ZipFile
 from calibre.utils.config import DynamicConfig
 from calibre.utils.logging import Log
-from calibre.ebooks.epub.output import EPUBOutput
 from calibre import guess_type, prints
+from calibre.ebooks.oeb.transforms.cover import CoverManager
 
-TITLEPAGE = EPUBOutput.TITLEPAGE_COVER.decode('utf-8')
+TITLEPAGE = CoverManager.SVG_TEMPLATE.decode('utf-8').replace(\
+        '__ar__', 'none').replace('__viewbox__', '0 0 600 800'
+        ).replace('__width__', '600').replace('__height__', '800')
 
 def character_count(html):
     '''
@@ -92,10 +94,10 @@ class EbookIterator(object):
         ext = re.sub(r'(x{0,1})htm(l{0,1})', 'html', ext)
         self.ebook_ext = ext
 
-    def search(self, text, index):
+    def search(self, text, index, backwards=False):
         text = text.lower()
         for i, path in enumerate(self.spine):
-            if i > index:
+            if (backwards and i < index) or (not backwards and i > index):
                 if text in open(path, 'rb').read().decode(path.encoding).lower():
                     return i
 
@@ -137,11 +139,18 @@ class EbookIterator(object):
                     if id != -1:
                         families = [unicode(f) for f in QFontDatabase.applicationFontFamilies(id)]
                         if family:
-                            family = family.group(1).strip().replace('"', '')
-                            bad_map[family] = families[0]
-                            if family not in families:
+                            family = family.group(1)
+                            specified_families = [x.strip().replace('"',
+                                '').replace("'", '') for x in family.split(',')]
+                            aliasing_ok = False
+                            for f in specified_families:
+                                bad_map[f] = families[0]
+                                if not aliasing_ok and f in families:
+                                    aliasing_ok = True
+
+                            if not aliasing_ok:
                                 prints('WARNING: Family aliasing not fully supported.')
-                                prints('\tDeclared family: %s not in actual families: %s'
+                                prints('\tDeclared family: %r not in actual families: %r'
                                         % (family, families))
                             else:
                                 prints('Loaded embedded font:', repr(family))
@@ -152,15 +161,19 @@ class EbookIterator(object):
                         prints('Substituting font family: %s -> %s'%(bad, good))
                         return match.group().replace(bad, '"%s"'%good)
 
+            from calibre.ebooks.chardet import force_encoding
             for csspath in css_files:
                 with open(csspath, 'r+b') as f:
                     css = f.read()
-                    css = font_family_pat.sub(prepend_embedded_font, css)
-                    f.seek(0)
-                    f.truncate()
-                    f.write(css)
+                    enc = force_encoding(css, False)
+                    css = css.decode(enc, 'replace')
+                    ncss = font_family_pat.sub(prepend_embedded_font, css)
+                    if ncss != css:
+                        f.seek(0)
+                        f.truncate()
+                        f.write(ncss.encode(enc))
 
-    def __enter__(self, processed=False):
+    def __enter__(self, processed=False, only_input_plugin=False):
         self.delete_on_exit = []
         self._tdir = TemporaryDirectory('_ebook_iter')
         self.base  = self._tdir.__enter__()
@@ -173,14 +186,19 @@ class EbookIterator(object):
             plumber.opts.no_process = True
 
         plumber.input_plugin.for_viewer = True
-        self.pathtoopf = plumber.input_plugin(open(plumber.input, 'rb'),
+        with plumber.input_plugin:
+            self.pathtoopf = plumber.input_plugin(open(plumber.input, 'rb'),
                 plumber.opts, plumber.input_fmt, self.log,
                 {}, self.base)
 
-        if processed or plumber.input_fmt.lower() in ('pdf', 'rb') and \
-                not hasattr(self.pathtoopf, 'manifest'):
-            self.pathtoopf = create_oebbook(self.log, self.pathtoopf, plumber.opts,
-                    plumber.input_plugin)
+        if not only_input_plugin:
+            if processed or plumber.input_fmt.lower() in ('pdb', 'pdf', 'rb') and \
+                    not hasattr(self.pathtoopf, 'manifest'):
+                if hasattr(self.pathtoopf, 'manifest'):
+                    self.pathtoopf = write_oebbook(self.pathtoopf, self.base)
+                self.pathtoopf = create_oebbook(self.log, self.pathtoopf, plumber.opts,
+                        plumber.input_plugin)
+
         if hasattr(self.pathtoopf, 'manifest'):
             self.pathtoopf = write_oebbook(self.pathtoopf, self.base)
 
@@ -207,11 +225,12 @@ class EbookIterator(object):
 
         cover = self.opf.cover
         if self.ebook_ext in ('lit', 'mobi', 'prc', 'opf') and cover:
-            cfile = os.path.join(os.path.dirname(self.spine[0]),
-                    'calibre_iterator_cover.html')
-            chtml = (TITLEPAGE%cover).encode('utf-8')
+            cfile = os.path.join(self.base, 'calibre_iterator_cover.html')
+            chtml = (TITLEPAGE%os.path.relpath(cover, self.base).replace(os.sep,
+                '/')).encode('utf-8')
             open(cfile, 'wb').write(chtml)
-            self.spine[0:0] = [SpineItem(cfile)]
+            self.spine[0:0] = [SpineItem(cfile,
+                mime_type='application/xhtml+xml')]
             self.delete_on_exit.append(cfile)
 
         if self.opf.path_to_html_toc is not None and \
