@@ -131,7 +131,7 @@ class OEBReader(object):
         stream = cStringIO.StringIO(etree.tostring(opf))
         mi = MetaInformation(OPF(stream))
         if not mi.language:
-            mi.language = get_lang()
+            mi.language = get_lang().replace('_', '-')
         self.oeb.metadata.add('language', mi.language)
         if not mi.title:
             mi.title = self.oeb.translate(__('Unknown'))
@@ -331,7 +331,10 @@ class OEBReader(object):
             id = child.get('id')
             klass = child.get('class', 'chapter')
 
-            po = int(child.get('playOrder', self.oeb.toc.next_play_order()))
+            try:
+                po = int(child.get('playOrder', self.oeb.toc.next_play_order()))
+            except:
+                po = self.oeb.toc.next_play_order()
 
             authorElement = xpath(child,
                     'descendant::calibre:meta[@name = "author"]')
@@ -356,7 +359,7 @@ class OEBReader(object):
             self._toc_from_navpoint(item, node, child)
 
     def _toc_from_ncx(self, item):
-        if item is None:
+        if (item is None) or (item.data is None):
             return False
         self.log.debug('Reading TOC from NCX...')
         ncx = item.data
@@ -467,6 +470,8 @@ class OEBReader(object):
         if item is None:
             return False
         ncx = item.data
+        if ncx is None:
+            return False
         ptargets = xpath(ncx, 'ncx:pageList/ncx:pageTarget')
         if not ptargets:
             return False
