@@ -25,10 +25,17 @@ from calibre.ebooks.oeb.base import XHTML, XHTML_NS, CSS_MIME, OEB_STYLES
 from calibre.ebooks.oeb.base import XPNSMAP, xpath, urlnormalize
 from calibre.ebooks.oeb.profile import PROFILES
 
-html_css = open(P('templates/html.css'), 'rb').read()
+_html_css_stylesheet = None
+
+def html_css_stylesheet():
+    global _html_css_stylesheet
+    if _html_css_stylesheet is None:
+        html_css = open(P('templates/html.css'), 'rb').read()
+        _html_css_stylesheet = cssutils.parseString(html_css)
+        _html_css_stylesheet.namespaces['h'] = XHTML_NS
+    return _html_css_stylesheet
+
 XHTML_CSS_NAMESPACE = '@namespace "%s";\n' % XHTML_NS
-HTML_CSS_STYLESHEET = cssutils.parseString(html_css)
-HTML_CSS_STYLESHEET.namespaces['h'] = XHTML_NS
 
 INHERITED = set(['azimuth', 'border-collapse', 'border-spacing',
                  'caption-side', 'color', 'cursor', 'direction', 'elevation',
@@ -96,6 +103,8 @@ class CSSSelector(etree.XPath):
             path = css_to_xpath(css)
         except UnicodeEncodeError: # Bug in css_to_xpath
             path = '/'
+        except NotImplementedError: # Probably a subselect like :hover
+            path = '/'
         path = self.LOCAL_NAME_RE.sub(r"local-name() = '", path)
         etree.XPath.__init__(self, path, namespaces=namespaces)
         self.css = css
@@ -118,12 +127,19 @@ class Stylizer(object):
         item = oeb.manifest.hrefs[path]
         basename = os.path.basename(path)
         cssname = os.path.splitext(basename)[0] + '.css'
-        stylesheets = [HTML_CSS_STYLESHEET]
+        stylesheets = [html_css_stylesheet()]
         head = xpath(tree, '/h:html/h:head')
         if head:
             head = head[0]
         else:
             head = []
+
+        # Add cssutils parsing profiles from output_profile
+        for profile in self.opts.output_profile.extra_css_modules:
+            cssutils.profile.addProfile(profile['name'],
+                                        profile['props'],
+                                        profile['macros'])
+
         parser = cssutils.CSSParser(fetcher=self._fetch_css_file,
                 log=logging.getLogger('calibre.css'))
         self.font_face_rules = []
@@ -174,6 +190,9 @@ class Stylizer(object):
         class_sel_pat = re.compile(r'\.[a-z]+', re.IGNORECASE)
         capital_sel_pat = re.compile(r'h|[A-Z]+')
         for _, _, cssdict, text, _ in rules:
+            fl = ':first-letter' in text
+            if fl:
+                text = text.replace(':first-letter', '')
             try:
                 selector = CSSSelector(text)
             except (AssertionError, ExpressionError, etree.XPathSyntaxError,
@@ -190,18 +209,31 @@ class Stylizer(object):
                     selector = CSSSelector(ntext)
                     matches = selector(tree)
 
-            if not matches and class_sel_pat.match(text):
+            if not matches and class_sel_pat.match(text) and text.lower() != text:
                 found = False
+                ltext = text.lower()
                 for x in tree.xpath('//*[@class]'):
-                    if text.lower().endswith('.'+x.get('class').lower()) and \
-                            text.lower() != text:
+                    if ltext.endswith('.'+x.get('class').lower()):
                         matches.append(x)
                         found = True
                 if found:
                     self.logger.warn('Ignoring case mismatches for CSS selector: %s in %s'
                         %(text, item.href))
-            for elem in matches:
-                self.style(elem)._update_cssdict(cssdict)
+            if fl:
+                from lxml.builder import ElementMaker
+                E = ElementMaker(namespace=XHTML_NS)
+                for elem in matches:
+                    for x in elem.iter():
+                        if x.text:
+                            span = E.span(x.text[0])
+                            span.tail = x.text[1:]
+                            x.text = None
+                            x.insert(0, span)
+                            self.style(span)._update_cssdict(cssdict)
+                            break
+            else:
+                for elem in matches:
+                    self.style(elem)._update_cssdict(cssdict)
         for elem in xpath(tree, '//h:img[@width or @height]'):
             base = elem.get('style', '').strip()
             if base:
@@ -318,8 +350,8 @@ class Stylizer(object):
         if text == 'inherit':
             style['text-align'] = 'inherit'
         else:
-            if text in ('left', 'justify'):
-                val = 'left' if self.opts.dont_justify else 'justify'
+            if text in ('left', 'justify') and self.opts.change_justification in ('left', 'justify'):
+                val = self.opts.change_justification
                 style['text-align'] = val
             else:
                 style['text-align'] = text
@@ -526,7 +558,7 @@ class Style(object):
                 base = parent.width
             else:
                 base = self._profile.width
-            if 'width' is self._element.attrib:
+            if 'width' in self._element.attrib:
                 width = self._element.attrib['width']
             elif 'width' in self._style:
                 width = self._style['width']
@@ -534,6 +566,8 @@ class Style(object):
                 result = base
             else:
                 result = self._unit_convert(width, base=base)
+            if isinstance(result, (unicode, str, bytes)):
+                result = self._profile.width
             self._width = result
         return self._width
 
@@ -547,7 +581,7 @@ class Style(object):
                 base = parent.height
             else:
                 base = self._profile.height
-            if 'height' is self._element.attrib:
+            if 'height' in self._element.attrib:
                 height = self._element.attrib['height']
             elif 'height' in self._style:
                 height = self._style['height']
@@ -555,6 +589,8 @@ class Style(object):
                 result = base
             else:
                 result = self._unit_convert(height, base=base)
+            if isinstance(result, (unicode, str, bytes)):
+                result = self._profile.height
             self._height = result
         return self._height
 
