@@ -138,8 +138,8 @@ class CSSFlattener(object):
                     float(self.context.margin_left))
             bs.append('margin-right : %fpt'%\
                     float(self.context.margin_right))
-            bs.append('text-align: '+ \
-                    ('left' if self.context.dont_justify else 'justify'))
+            if self.context.change_justification != 'original':
+                bs.append('text-align: '+ self.context.change_justification)
             body.set('style', '; '.join(bs))
             stylizer = Stylizer(html, item.href, self.oeb, self.context, profile,
                     user_css=self.context.extra_css,
@@ -164,7 +164,10 @@ class CSSFlattener(object):
             body = html.find(XHTML('body'))
             fsize = self.context.source.fbase
             self.baseline_node(body, stylizer, sizes, fsize)
-        sbase = max(sizes.items(), key=operator.itemgetter(1))[0]
+        try:
+            sbase = max(sizes.items(), key=operator.itemgetter(1))[0]
+        except:
+            sbase = 12.0
         self.oeb.logger.info(
             "Source base font size is %0.05fpt" % sbase)
         return sbase
@@ -198,6 +201,11 @@ class CSSFlattener(object):
         tag = barename(node.tag)
         style = stylizer.style(node)
         cssdict = style.cssdict()
+        try:
+            font_size = style['font-size']
+        except:
+            font_size = self.sbase if self.sbase is not None else \
+                self.context.source.fbase
         if 'align' in node.attrib:
             cssdict['text-align'] = node.attrib['align']
             del node.attrib['align']
@@ -216,13 +224,16 @@ class CSSFlattener(object):
                             esize = 1
                         if esize > 7:
                             esize = 7
-                        cssdict['font-size'] = fnums[esize]
+                        font_size = fnums[esize]
                     else:
                         try:
-                            cssdict['font-size'] = fnums[force_int(size)]
+                            font_size = fnums[force_int(size)]
                         except:
-                            cssdict['font-size'] = fnums[3]
+                            font_size = fnums[3]
+                    cssdict['font-size'] = '%.1fpt'%font_size
                 del node.attrib['size']
+            if 'face' in node.attrib:
+                del node.attrib['face']
         if 'color' in node.attrib:
             cssdict['color'] = node.attrib['color']
             del node.attrib['color']
@@ -241,7 +252,7 @@ class CSSFlattener(object):
                 cssdict['font-size'] = '%0.5fem'%(fsize/psize)
                 psize = fsize
             elif 'font-size' in cssdict or tag == 'body':
-                fsize = self.fmap[style['font-size']]
+                fsize = self.fmap[font_size]
                 cssdict['font-size'] = "%0.5fem" % (fsize / psize)
                 psize = fsize
         if cssdict:
