@@ -17,13 +17,13 @@ __docformat__ = 'restructuredtext en'
 
 HTML_TEMPLATE = u'<html><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8"/><title>%s</title></head><body>\n%s\n</body></html>'
 
-def convert_basic(txt, title=''):
-    lines = []
+def convert_basic(txt, title='', epub_split_size_kb=0):
     # Strip whitespace from the beginning and end of the line. Also replace
     # all line breaks with \n.
-    for line in txt.splitlines():
-        lines.append(line.strip())
-    txt = '\n'.join(lines)
+    txt = '\n'.join([line.strip() for line in txt.splitlines()])
+
+    # Condense redundant spaces
+    txt = re.sub('[ ]{2,}', ' ', txt)
 
     # Remove blank lines from the beginning and end of the document.
     txt = re.sub('^\s+(?=.)', '', txt)
@@ -31,6 +31,15 @@ def convert_basic(txt, title=''):
     # Remove excessive line breaks.
     txt = re.sub('\n{3,}', '\n\n', txt)
 
+    #Takes care if there is no point to split
+    if epub_split_size_kb > 0:
+        length_byte = len(txt.encode('utf-8'))
+        #Calculating the average chunk value for easy splitting as EPUB (+2 as a safe margin)
+        chunk_size = long(length_byte / (int(length_byte / (epub_split_size_kb * 1024) ) + 2 ))
+        #if there are chunks with a superior size then go and break
+        if (len(filter(lambda x: len(x.encode('utf-8')) > chunk_size, txt.split('\n\n')))) :
+            txt = u'\n\n'.join([split_string_separator(line, chunk_size) for line in txt.split('\n\n')])
+
     lines = []
     # Split into paragraphs based on having a blank line between text.
     for line in txt.split('\n\n'):
@@ -56,6 +65,11 @@ def separate_paragraphs_print_formatted(txt):
     txt = re.sub('(?miu)^(\t+|[ ]{2,})(?=.)', '\n\t', txt)
     return txt
 
+def preserve_spaces(txt):
+    txt = txt.replace(' ', '&nbsp;')
+    txt = txt.replace('\t', '&#09;')
+    return txt
+
 def opf_writer(path, opf_name, manifest, spine, mi):
     opf = OPFCreator(path, mi)
     opf.create_manifest(manifest)
@@ -63,3 +77,10 @@ def opf_writer(path, opf_name, manifest, spine, mi):
     with open(os.path.join(path, opf_name), 'wb') as opffile:
         opf.render(opffile)
 
+def split_string_separator(txt, size) :
+    if len(txt.encode('utf-8')) > size:
+        txt = u''.join([re.sub(u'\.(?P<ends>[^.]*)$', u'.\n\n\g<ends>',
+            txt[i:i+size], 1) for i in
+            xrange(0, len(txt.encode('utf-8')), size)])
+    return txt
+
