@@ -1,28 +1,30 @@
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 """ The GUI """
-import os
+import os, sys
 from threading import RLock
 
-from PyQt4.QtCore import QVariant, QFileInfo, QObject, SIGNAL, QBuffer, Qt, QSize, \
+from PyQt4.Qt import QVariant, QFileInfo, QObject, SIGNAL, QBuffer, Qt, QSize, \
                          QByteArray, QTranslator, QCoreApplication, QThread, \
-                         QEvent, QTimer, pyqtSignal
-from PyQt4.QtGui import QFileDialog, QMessageBox, QPixmap, QFileIconProvider, \
-                        QIcon, QTableView, QApplication, QDialog, QPushButton
+                         QEvent, QTimer, pyqtSignal, QDate, QDesktopServices, \
+                         QFileDialog, QMessageBox, QPixmap, QFileIconProvider, \
+                         QIcon, QApplication, QDialog, QPushButton, QUrl
 
 ORG_NAME = 'KovidsBrain'
 APP_UID  = 'libprs500'
-from calibre import islinux, iswindows, isosx
+from calibre.constants import islinux, iswindows, isosx, isfreebsd, isfrozen
 from calibre.utils.config import Config, ConfigProxy, dynamic, JSONConfig
 from calibre.utils.localization import set_qt_translator
 from calibre.ebooks.metadata.meta import get_metadata, metadata_from_formats
 from calibre.ebooks.metadata import MetaInformation
+from calibre.utils.date import UNDEFINED_DATE
 
 gprefs = JSONConfig('gui')
 
 NONE = QVariant() #: Null value to return from the data function of item models
+UNDEFINED_QDATE = QDate(UNDEFINED_DATE)
 
-ALL_COLUMNS = ['title', 'authors', 'size', 'timestamp', 'rating', 'publisher',
+ALL_COLUMNS = ['title', 'ondevice', 'authors', 'size', 'timestamp', 'rating', 'publisher',
         'tags', 'series', 'pubdate']
 
 def _config():
@@ -41,8 +43,8 @@ def _config():
               help=_('Notify when a new version is available'))
     c.add_opt('use_roman_numerals_for_series_number', default=True,
               help=_('Use Roman numerals for series number'))
-    c.add_opt('sort_by_popularity', default=False,
-              help=_('Sort tags list by popularity'))
+    c.add_opt('sort_tags_by', default='name',
+              help=_('Sort tags list by name, popularity, or rating'))
     c.add_opt('cover_flow_queue_length', default=6,
               help=_('Number of covers to show in the cover browsing mode'))
     c.add_opt('LRF_conversion_defaults', default=[],
@@ -95,7 +97,14 @@ def _config():
             help=_('Overwrite author and title with new metadata'))
     c.add_opt('enforce_cpu_limit', default=True,
             help=_('Limit max simultaneous jobs to number of CPUs'))
-
+    c.add_opt('tag_browser_hidden_categories', default=set(),
+            help=_('tag browser categories not to display'))
+    c.add_opt('gui_layout', choices=['wide', 'narrow'],
+            help=_('The layout of the user interface'), default='wide')
+    c.add_opt('show_avg_rating', default=True,
+            help=_('Show the average rating per item indication in the tag browser'))
+    c.add_opt('disable_animations', default=False,
+            help=_('Disable UI animations'))
     return ConfigProxy(c)
 
 config = _config()
@@ -122,6 +131,17 @@ def available_width():
     desktop       = QCoreApplication.instance().desktop()
     return desktop.availableGeometry().width()
 
+_is_widescreen = None
+
+def is_widescreen():
+    global _is_widescreen
+    if _is_widescreen is None:
+        try:
+            _is_widescreen = float(available_width())/available_height() > 1.4
+        except:
+            _is_widescreen = False
+    return _is_widescreen
+
 def extension(path):
     return os.path.splitext(path)[1][1:].lower()
 
@@ -182,29 +202,38 @@ class MessageBox(QMessageBox):
 
 
 
-def warning_dialog(parent, title, msg, det_msg='', show=False):
+def warning_dialog(parent, title, msg, det_msg='', show=False,
+        show_copy_button=True):
     d = MessageBox(QMessageBox.Warning, 'WARNING: '+title, msg, QMessageBox.Ok,
                     parent, det_msg)
     d.setEscapeButton(QMessageBox.Ok)
     d.setIconPixmap(QPixmap(I('dialog_warning.svg')))
+    if not show_copy_button:
+        d.cb.setVisible(False)
     if show:
         return d.exec_()
     return d
 
-def error_dialog(parent, title, msg, det_msg='', show=False):
+def error_dialog(parent, title, msg, det_msg='', show=False,
+        show_copy_button=True):
     d = MessageBox(QMessageBox.Critical, 'ERROR: '+title, msg, QMessageBox.Ok,
                     parent, det_msg)
     d.setIconPixmap(QPixmap(I('dialog_error.svg')))
     d.setEscapeButton(QMessageBox.Ok)
+    if not show_copy_button:
+        d.cb.setVisible(False)
     if show:
         return d.exec_()
     return d
 
-def question_dialog(parent, title, msg, det_msg=''):
+def question_dialog(parent, title, msg, det_msg='', show_copy_button=True):
     d = MessageBox(QMessageBox.Question, title, msg, QMessageBox.Yes|QMessageBox.No,
                     parent, det_msg)
     d.setIconPixmap(QPixmap(I('dialog_information.svg')))
     d.setEscapeButton(QMessageBox.No)
+    if not show_copy_button:
+        d.cb.setVisible(False)
+
     return d.exec_() == QMessageBox.Yes
 
 def info_dialog(parent, title, msg, det_msg='', show=False):
@@ -216,28 +245,10 @@ def info_dialog(parent, title, msg, det_msg='', show=False):
     return d
 
 
-def qstring_to_unicode(q):
-    return unicode(q)
-
-def human_readable(size):
-    """ Convert a size in bytes into a human readable form """
-    divisor, suffix = 1, "B"
-    if size < 1024*1024:
-        divisor, suffix = 1024., "KB"
-    elif size < 1024*1024*1024:
-        divisor, suffix = 1024*1024, "MB"
-    elif size < 1024*1024*1024*1024:
-        divisor, suffix = 1024*1024*1024, "GB"
-    size = str(float(size)/divisor)
-    if size.find(".") > -1:
-        size = size[:size.find(".")+2]
-    if size.endswith('.0'):
-        size = size[:-2]
-    return size + " " + suffix
 
 class Dispatcher(QObject):
     '''Convenience class to ensure that a function call always happens in the
-    thread the reciver was created in.'''
+    thread the receiver was created in.'''
     dispatch_signal = pyqtSignal(object, object)
 
     def __init__(self, func):
@@ -286,25 +297,6 @@ class GetMetadata(QObject):
             mi = MetaInformation('', [_('Unknown')])
         self.emit(SIGNAL('metadata(PyQt_PyObject, PyQt_PyObject)'), id, mi)
 
-class TableView(QTableView):
-
-    def __init__(self, parent):
-        QTableView.__init__(self, parent)
-        self.read_settings()
-
-    def read_settings(self):
-        self.cw = dynamic[self.__class__.__name__+'column widths']
-
-    def write_settings(self):
-        dynamic[self.__class__.__name__+'column widths'] = \
-         tuple([int(self.columnWidth(i)) for i in range(self.model().columnCount(None))])
-
-    def restore_column_widths(self):
-        if self.cw and len(self.cw):
-            for i in range(len(self.cw)):
-                self.setColumnWidth(i, self.cw[i])
-            return True
-
 class FileIconProvider(QFileIconProvider):
 
     ICONS = {
@@ -326,14 +318,20 @@ class FileIconProvider(QFileIconProvider):
              'lrf'     : 'lrf',
              'lrx'     : 'lrx',
              'pdf'     : 'pdf',
+             'pdr'     : 'zero',
              'rar'     : 'rar',
              'zip'     : 'zip',
              'txt'     : 'txt',
              'prc'     : 'mobi',
              'azw'     : 'mobi',
              'mobi'    : 'mobi',
+             'mbp'     : 'zero',
+             'azw1'    : 'mobi',
+             'tpz'     : 'mobi',
+             'tan'     : 'zero',
              'epub'    : 'epub',
              'fb2'     : 'fb2',
+             'rtf'     : 'rtf',
              }
 
     def __init__(self):
@@ -373,7 +371,7 @@ class FileIconProvider(QFileIconProvider):
         if fileinfo.isDir():
             key = 'dir'
         else:
-            ext = qstring_to_unicode(fileinfo.completeSuffix()).lower()
+            ext = unicode(fileinfo.completeSuffix()).lower()
             key = self.key_from_ext(ext)
         return self.cached_icon(key)
 
@@ -405,6 +403,7 @@ class FileDialog(QObject):
                        modal = True,
                        name = '',
                        mode = QFileDialog.ExistingFiles,
+                       default_dir='~'
                        ):
         QObject.__init__(self)
         ftext = ''
@@ -423,9 +422,10 @@ class FileDialog(QObject):
         self.selected_files = None
         self.fd = None
 
-        initial_dir = dynamic.get(self.dialog_name, os.path.expanduser('~'))
+        initial_dir = dynamic.get(self.dialog_name,
+                os.path.expanduser(default_dir))
         if not isinstance(initial_dir, basestring):
-            initial_dir = os.path.expanduser('~')
+            initial_dir = os.path.expanduser(default_dir)
         self.selected_files = []
         if mode == QFileDialog.AnyFile:
             f = unicode(QFileDialog.getSaveFileName(parent, title, initial_dir, ftext, ""))
@@ -456,13 +456,14 @@ class FileDialog(QObject):
 
     def get_files(self):
         if self.selected_files is None:
-            return tuple(os.path.abspath(qstring_to_unicode(i)) for i in self.fd.selectedFiles())
+            return tuple(os.path.abspath(unicode(i)) for i in self.fd.selectedFiles())
         return tuple(self.selected_files)
 
 
-def choose_dir(window, name, title):
-    fd = FileDialog(title, [], False, window, name=name,
-                    mode=QFileDialog.DirectoryOnly)
+def choose_dir(window, name, title, default_dir='~'):
+    fd = FileDialog(title=title, filters=[], add_all_files_filter=False,
+            parent=window, name=name, mode=QFileDialog.DirectoryOnly,
+            default_dir=default_dir)
     dir = fd.get_files()
     if dir:
         return dir[0]
@@ -506,7 +507,7 @@ def pixmap_to_data(pixmap, format='JPEG'):
     buf = QBuffer(ba)
     buf.open(QBuffer.WriteOnly)
     pixmap.save(buf, format)
-    return str(ba.data())
+    return bytes(ba.data())
 
 class ResizableDialog(QDialog):
 
@@ -547,6 +548,7 @@ class Application(QApplication):
                         border-radius: 10px;
                         opacity: 200;
                         background-color: #e1e1ff;
+                        color: black;
                     }
             ''')
 
@@ -577,9 +579,25 @@ class Application(QApplication):
 
 _store_app = None
 
+def open_url(qurl):
+    paths = os.environ.get('LD_LIBRARY_PATH',
+                '').split(os.pathsep)
+    paths = [x for x in paths if x]
+    if isfrozen and islinux and paths:
+        npaths = [x for x in paths if x != sys.frozen_path]
+        os.environ['LD_LIBRARY_PATH'] = os.pathsep.join(npaths)
+    QDesktopServices.openUrl(qurl)
+    if isfrozen and islinux and paths:
+        os.environ['LD_LIBRARY_PATH'] = os.pathsep.join(paths)
+
+
+def open_local_file(path):
+    url = QUrl.fromLocalFile(path)
+    open_url(url)
+
 def is_ok_to_use_qt():
     global gui_thread, _store_app
-    if islinux and ':' not in os.environ.get('DISPLAY', ''):
+    if (islinux or isfreebsd) and ':' not in os.environ.get('DISPLAY', ''):
         return False
     if _store_app is None and QApplication.instance() is None:
         _store_app = QApplication([])
@@ -587,6 +605,11 @@ def is_ok_to_use_qt():
         gui_thread = QThread.currentThread()
     return gui_thread is QThread.currentThread()
 
+def is_gui_thread():
+    global gui_thread
+    return gui_thread is QThread.currentThread()
+
+
 def find_forms(srcdir):
     base = os.path.join(srcdir, 'calibre', 'gui2')
     forms = []
