@@ -14,6 +14,7 @@ class PluginWidget(Widget, Ui_Form):
 
     def __init__(self, parent, get_option, get_help, db=None, book_id=None):
         Widget.__init__(self, parent, 'txt_input',
-            ['single_line_paras', 'print_formatted_paras', 'markdown', 'markdown_disable_toc'])
+            ['single_line_paras', 'print_formatted_paras', 'markdown',
+                'markdown_disable_toc', 'preserve_spaces'])
         self.db, self.book_id = db, book_id
         self.initialize_options(get_option, get_help, db, book_id)
