@@ -3,16 +3,15 @@ __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
 __docformat__ = 'restructuredtext en'
 
-'''
-'''
-import textwrap, os
+import textwrap, os, re
 
-from PyQt4.QtCore import QCoreApplication, SIGNAL, QModelIndex, QUrl, QTimer, Qt
-from PyQt4.QtGui import QDialog, QPixmap, QGraphicsScene, QIcon, QDesktopServices
+from PyQt4.QtCore import QCoreApplication, SIGNAL, QModelIndex, QTimer, Qt
+from PyQt4.QtGui import QDialog, QPixmap, QGraphicsScene, QIcon
 
 from calibre.gui2.dialogs.book_info_ui import Ui_BookInfo
-from calibre.gui2 import dynamic
+from calibre.gui2 import dynamic, open_local_file
 from calibre import fit_image
+from calibre.library.comments import comments_to_html
 
 class BookInfo(QDialog, Ui_BookInfo):
 
@@ -50,12 +49,12 @@ class BookInfo(QDialog, Ui_BookInfo):
 
     def open_book_path(self, path):
         if os.sep in unicode(path):
-            QDesktopServices.openUrl(QUrl('file:'+path))
+            open_local_file(path)
         else:
             format = unicode(path)
             path = self.view.model().db.format_abspath(self.current_row, format)
             if path is not None:
-                QDesktopServices.openUrl(QUrl('file:'+path))
+                open_local_file(path)
 
 
     def next(self):
@@ -97,8 +96,14 @@ class BookInfo(QDialog, Ui_BookInfo):
         info = self.view.model().get_book_info(row)
         self.setWindowTitle(info[_('Title')])
         self.title.setText('<b>'+info.pop(_('Title')))
-        self.comments.setText(info.pop(_('Comments'), ''))
-
+        comments = info.pop(_('Comments'), '')
+        if comments:
+            comments = comments_to_html(comments)
+        if re.search(r'<[a-zA-Z]+>', comments) is None:
+            lines = comments.splitlines()
+            lines = [x if x.strip() else '<br><br>' for x in lines]
+            comments = '\n'.join(lines)
+        self.comments.setText('<div>%s</div>' % comments)
         cdata = info.pop('cover', '')
         self.cover_pixmap = QPixmap.fromImage(cdata)
         self.resize_cover()
@@ -116,7 +121,9 @@ class BookInfo(QDialog, Ui_BookInfo):
                 f = f.strip()
                 info[_('Formats')] += '<a href="%s">%s</a>, '%(f,f)
         for key in info.keys():
+            if key == 'id': continue
             txt  = info[key]
-            txt  = u'<br />\n'.join(textwrap.wrap(txt, 120))
+            if key != _('Path'):
+                txt  = u'<br />\n'.join(textwrap.wrap(txt, 120))
             rows += u'<tr><td><b>%s:</b></td><td>%s</td></tr>'%(key, txt)
         self.text.setText(u'<table>'+rows+'</table>')
