@@ -11,10 +11,10 @@ import re
 import time
 import traceback
 
-from PyQt4.Qt import SIGNAL, QObject, QCoreApplication, Qt, QTimer, QThread, QDate, \
+from PyQt4.Qt import SIGNAL, QObject, Qt, QTimer, QThread, QDate, \
     QPixmap, QListWidgetItem, QDialog
 
-from calibre.gui2 import qstring_to_unicode, error_dialog, file_icon_provider, \
+from calibre.gui2 import error_dialog, file_icon_provider, dynamic, \
                            choose_files, choose_images, ResizableDialog, \
                            warning_dialog
 from calibre.gui2.dialogs.metadata_single_ui import Ui_MetadataSingleDialog
@@ -22,15 +22,15 @@ from calibre.gui2.dialogs.fetch_metadata import FetchMetadata
 from calibre.gui2.dialogs.tag_editor import TagEditor
 from calibre.gui2.widgets import ProgressIndicator
 from calibre.ebooks import BOOK_EXTENSIONS
-from calibre.ebooks.metadata import authors_to_sort_string, string_to_authors, \
+from calibre.ebooks.metadata import string_to_authors, \
         authors_to_string, check_isbn
 from calibre.ebooks.metadata.library_thing import cover_from_isbn
-from calibre import islinux
 from calibre.ebooks.metadata.meta import get_metadata
 from calibre.utils.config import prefs, tweaks
 from calibre.utils.date import qt_to_dt
 from calibre.customize.ui import run_plugins_on_import, get_isbndb_key
 from calibre.gui2.dialogs.config.social import SocialMetadata
+from calibre.gui2.custom_column_widgets import populate_metadata_page
 
 class CoverFetcher(QThread):
 
@@ -89,7 +89,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
     COVER_FETCH_TIMEOUT = 240 # seconds
 
     def do_reset_cover(self, *args):
-        pix = QPixmap(I('book.svg'))
+        pix = QPixmap(I('default_cover.svg'))
         self.cover.setPixmap(pix)
         self.cover_changed = True
         self.cover_data = None
@@ -103,7 +103,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         if _file:
             _file = os.path.abspath(_file)
             if not os.access(_file, os.R_OK):
-                d = error_dialog(self.window, _('Cannot read'),
+                d = error_dialog(self, _('Cannot read'),
                         _('You do not have permission to read the file: ') + _file)
                 d.exec_()
                 return
@@ -112,14 +112,14 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 cf = open(_file, "rb")
                 cover = cf.read()
             except IOError, e:
-                d = error_dialog(self.window, _('Error reading file'),
+                d = error_dialog(self, _('Error reading file'),
                         _("<p>There was an error reading from file: <br /><b>") + _file + "</b></p><br />"+str(e))
                 d.exec_()
             if cover:
                 pix = QPixmap()
                 pix.loadFromData(cover)
                 if pix.isNull():
-                    d = error_dialog(self.window,
+                    d = error_dialog(self,
                         _("Not a valid picture"),
                             _file + _(" is not a valid picture"))
                     d.exec_()
@@ -162,7 +162,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.formats_changed = True
             added = True
         if bad_perms:
-            error_dialog(self.window, _('No permission'),
+            error_dialog(self, _('No permission'),
                     _('You do not have '
                 'permission to read the following files:'),
                 det_msg='\n'.join(bad_perms), show=True)
@@ -180,27 +180,34 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.formats_changed = True
 
     def get_selected_format_metadata(self):
-        row = self.formats.currentRow()
-        fmt = self.formats.item(row)
-        if fmt is None:
-            if self.formats.count() == 1:
-                fmt = self.formats.item(0)
-            if fmt is None:
-                error_dialog(self, _('No format selected'),
-                    _('No format selected')).exec_()
-                return None, None
-        ext = fmt.ext.lower()
-        if fmt.path is None:
-            stream = self.db.format(self.row, ext, as_file=True)
-        else:
-            stream = open(fmt.path, 'r+b')
+        old = prefs['read_file_metadata']
+        if not old:
+            prefs['read_file_metadata'] = True
         try:
-            mi = get_metadata(stream, ext)
-            return mi, ext
-        except:
-            error_dialog(self, _('Could not read metadata'),
-                         _('Could not read metadata from %s format')%ext).exec_()
-        return None, None
+            row = self.formats.currentRow()
+            fmt = self.formats.item(row)
+            if fmt is None:
+                if self.formats.count() == 1:
+                    fmt = self.formats.item(0)
+                if fmt is None:
+                    error_dialog(self, _('No format selected'),
+                        _('No format selected')).exec_()
+                    return None, None
+            ext = fmt.ext.lower()
+            if fmt.path is None:
+                stream = self.db.format(self.row, ext, as_file=True)
+            else:
+                stream = open(fmt.path, 'r+b')
+            try:
+                mi = get_metadata(stream, ext)
+                return mi, ext
+            except:
+                error_dialog(self, _('Could not read metadata'),
+                            _('Could not read metadata from %s format')%ext).exec_()
+            return None, None
+        finally:
+            if old != prefs['read_file_metadata']:
+                prefs['read_file_metadata'] = old
 
     def set_metadata_from_format(self):
         mi, ext = self.get_selected_format_metadata()
@@ -231,7 +238,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             if mi.series_index is not None:
                 self.series_index.setValue(float(mi.series_index))
         if mi.comments and mi.comments.strip():
-            self.comments.setText(mi.comments)
+            self.comments.setPlainText(mi.comments)
 
 
     def set_cover(self):
@@ -292,6 +299,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.connect(self.__abort_button, SIGNAL('clicked()'),
                     self.do_cancel_all)
         self.splitter.setStretchFactor(100, 1)
+        self.read_state()
         self.db = db
         self.pi = ProgressIndicator(self)
         self.accepted_callback = accepted_callback
@@ -302,8 +310,10 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         self.formats.setAcceptDrops(True)
         self.cover_changed = False
         self.cpixmap = None
-        self.cover.setAcceptDrops(True)
         self.pubdate.setMinimumDate(QDate(100,1,1))
+        pubdate_format = tweaks['gui_pubdate_display_format']
+        if pubdate_format is not None:
+            self.pubdate.setDisplayFormat(pubdate_format)
         self.date.setMinimumDate(QDate(100,1,1))
 
         self.connect(self.cover, SIGNAL('cover_changed(PyQt_PyObject)'), self.cover_dropped)
@@ -345,7 +355,8 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         aus = self.db.author_sort(row)
         self.author_sort.setText(aus if aus else '')
         tags = self.db.tags(row)
-        self.tags.setText(', '.join(tags.split(',')) if tags else '')
+        self.original_tags = ', '.join(tags.split(',')) if tags else ''
+        self.tags.setText(self.original_tags)
         self.tags.update_tags_cache(self.db.all_tags())
         rating = self.db.rating(row)
         if rating > 0:
@@ -386,18 +397,35 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         self.series.lineEdit().editingFinished.connect(self.increment_series_index)
 
         self.show()
-        height_of_rest = self.frameGeometry().height() - self.cover.height()
-        width_of_rest  = self.frameGeometry().width() - self.cover.width()
-        ag = QCoreApplication.instance().desktop().availableGeometry(self)
-        self.cover.MAX_HEIGHT = ag.height()-(25 if islinux else 0)-height_of_rest
-        self.cover.MAX_WIDTH = ag.width()-(25 if islinux else 0)-width_of_rest
+        pm = QPixmap()
         if cover:
-            pm = QPixmap()
             pm.loadFromData(cover)
-            if not pm.isNull():
-                self.cover.setPixmap(pm)
+        if pm.isNull():
+            pm = QPixmap(I('default_cover.svg'))
+        else:
             self.cover_data = cover
+        self.cover.setPixmap(pm)
         self.original_series_name = unicode(self.series.text()).strip()
+        if len(db.custom_column_label_map) == 0:
+            self.central_widget.tabBar().setVisible(False)
+        else:
+            self.create_custom_column_editors()
+
+    def create_custom_column_editors(self):
+        w = self.central_widget.widget(1)
+        layout = w.layout()
+        self.custom_column_widgets, self.__cc_spacers = \
+                    populate_metadata_page(layout, self.db, self.id,
+                                           parent=w, bulk=False, two_column=True)
+        self.__custom_col_layouts = [layout]
+        ans = self.custom_column_widgets
+        for i in range(len(ans)-1):
+            if len(ans[i+1].widgets) == 2:
+                w.setTabOrder(ans[i].widgets[-1], ans[i+1].widgets[1])
+            else:
+                w.setTabOrder(ans[i].widgets[-1], ans[i+1].widgets[0])
+            for c in range(2, len(ans[i].widgets), 2):
+                w.setTabOrder(ans[i].widgets[c-1], ans[i].widgets[c+1])
 
     def validate_isbn(self, isbn):
         isbn = unicode(isbn).strip()
@@ -421,7 +449,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         au = unicode(self.authors.text())
         au = re.sub(r'\s+et al\.$', '', au)
         authors = string_to_authors(au)
-        self.author_sort.setText(authors_to_sort_string(authors))
+        self.author_sort.setText(self.db.author_sort_from_authors(authors))
 
     def swap_title_author(self):
         title = self.title.text()
@@ -489,6 +517,10 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.publisher.setCurrentIndex(idx)
 
     def edit_tags(self):
+        if self.tags.text() != self.original_tags:
+            error_dialog(self, _('Cannot use tag editor'),
+                _('The tags editor cannot be used if you have modified the tags')).exec_()
+            return
         d = TagEditor(self, self.db, self.row)
         d.exec_()
         if d.result() == QDialog.Accepted:
@@ -552,9 +584,12 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
     def fetch_metadata(self):
         isbn   = re.sub(r'[^0-9a-zA-Z]', '', unicode(self.isbn.text()))
-        title  = qstring_to_unicode(self.title.text())
-        author = string_to_authors(unicode(self.authors.text()))[0]
-        publisher = qstring_to_unicode(self.publisher.currentText())
+        title  = unicode(self.title.text())
+        try:
+            author = string_to_authors(unicode(self.authors.text()))[0]
+        except:
+            author = ''
+        publisher = unicode(self.publisher.currentText())
         if isbn or title or author or publisher:
             d = FetchMetadata(self, isbn, title, author, publisher, self.timeout)
             self._fetch_metadata_scope = d
@@ -587,7 +622,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                             prefix = unicode(self.comments.toPlainText())
                             if prefix:
                                 prefix += '\n'
-                            self.comments.setText(prefix + summ)
+                            self.comments.setPlainText(prefix + summ)
                         if book.rating is not None:
                             self.rating.setValue(int(book.rating))
                         if book.tags:
@@ -620,12 +655,12 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
     def remove_unused_series(self):
         self.db.remove_unused_series()
-        idx = qstring_to_unicode(self.series.currentText())
+        idx = unicode(self.series.currentText())
         self.series.clear()
         self.initialize_series()
         if idx:
             for i in range(self.series.count()):
-                if qstring_to_unicode(self.series.itemText(i)) == idx:
+                if unicode(self.series.itemText(i)) == idx:
                     self.series.setCurrentIndex(i)
                     break
 
@@ -645,26 +680,28 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.db.set_isbn(self.id,
                     re.sub(r'[^0-9a-zA-Z]', '', unicode(self.isbn.text())), notify=False)
             self.db.set_rating(self.id, 2*self.rating.value(), notify=False)
-            self.db.set_publisher(self.id, qstring_to_unicode(self.publisher.currentText()), notify=False)
+            self.db.set_publisher(self.id, unicode(self.publisher.currentText()), notify=False)
             self.db.set_tags(self.id, [x.strip() for x in
                 unicode(self.tags.text()).split(',')], notify=False)
             self.db.set_series(self.id,
                     unicode(self.series.currentText()).strip(), notify=False)
             self.db.set_series_index(self.id, self.series_index.value(), notify=False)
-            self.db.set_comment(self.id, qstring_to_unicode(self.comments.toPlainText()), notify=False)
+            self.db.set_comment(self.id, unicode(self.comments.toPlainText()), notify=False)
             d = self.pubdate.date()
             d = qt_to_dt(d)
-            self.db.set_pubdate(self.id, d)
+            self.db.set_pubdate(self.id, d, notify=False)
             d = self.date.date()
             d = qt_to_dt(d)
             if d.date() != self.orig_timestamp.date():
-                self.db.set_timestamp(self.id, d)
+                self.db.set_timestamp(self.id, d, notify=False)
 
             if self.cover_changed:
                 if self.cover_data is not None:
                     self.db.set_cover(self.id, self.cover_data)
                 else:
                     self.db.remove_cover(self.id)
+            for w in getattr(self, 'custom_column_widgets', []):
+                w.commit(self.id)
         except IOError, err:
             if err.errno == 13: # Permission denied
                 fname = err.filename if err.filename else 'file'
@@ -672,7 +709,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                         _('Could not open %s. Is it being used by another'
                         ' program?')%fname, show=True)
             raise
-
+        self.save_state()
         QDialog.accept(self)
         if callable(self.accepted_callback):
             self.accepted_callback(self.id)
@@ -684,3 +721,16 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             cf.wait()
 
         QDialog.reject(self, *args)
+
+    def read_state(self):
+        wg = dynamic.get('metasingle_window_geometry', None)
+        ss = dynamic.get('metasingle_splitter_state', None)
+        if wg is not None:
+          self.restoreGeometry(wg)
+        if ss is not None:
+          self.splitter.restoreState(ss)
+
+    def save_state(self):
+        dynamic.set('metasingle_window_geometry', bytes(self.saveGeometry()))
+        dynamic.set('metasingle_splitter_state',
+                bytes(self.splitter.saveState()))
