@@ -18,6 +18,7 @@ from calibre.gui2 import config as gconf, error_dialog
 from calibre.web.feeds.recipes.model import RecipeModel
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.date import utcnow
+from calibre.utils.network import internet_connected
 
 class SchedulerDialog(QDialog, Ui_Dialog):
 
@@ -28,10 +29,10 @@ class SchedulerDialog(QDialog, Ui_Dialog):
         self.recipe_model.do_refresh()
 
         self.search = SearchBox2(self)
+        self.search.setMinimumContentsLength(25)
         self.search.initialize('scheduler_search_history')
         self.recipe_box.layout().insertWidget(0, self.search)
-        self.connect(self.search, SIGNAL('search(PyQt_PyObject,PyQt_PyObject)'),
-                self.recipe_model.search)
+        self.search.search.connect(self.recipe_model.search)
         self.connect(self.recipe_model,  SIGNAL('searched(PyQt_PyObject)'),
                 self.search.search_done)
         self.connect(self.recipe_model,  SIGNAL('searched(PyQt_PyObject)'),
@@ -204,6 +205,7 @@ class Scheduler(QObject):
 
     def __init__(self, parent, db):
         QObject.__init__(self, parent)
+        self.internet_connection_failed = False
         self._parent = parent
         self.recipe_model = RecipeModel(db)
         self.lock = QMutex(QMutex.Recursive)
@@ -217,6 +219,10 @@ class Scheduler(QObject):
         self.cac = QAction(QIcon(I('user_profile.svg')), _('Add a custom news source'), self)
         self.connect(self.cac, SIGNAL('triggered(bool)'), self.customize_feeds)
         self.news_menu.addAction(self.cac)
+        self.news_menu.addSeparator()
+        self.all_action = self.news_menu.addAction(
+                _('Download all scheduled new sources'),
+                self.download_all_scheduled)
 
         self.timer = QTimer(self)
         self.timer.start(int(self.INTERVAL * 60000))
@@ -301,14 +307,29 @@ class Scheduler(QObject):
         if urn is not None:
             return self.download(urn)
         for urn in self.recipe_model.scheduled_urns():
-            self.download(urn)
+            if not self.download(urn):
+                break
+
+    def download_all_scheduled(self):
+        self.download_clicked(None)
 
     def download(self, urn):
         self.lock.lock()
+        if not internet_connected():
+            if not self.internet_connection_failed:
+                self.internet_connection_failed = True
+                d = error_dialog(self._parent, _('No internet connection'),
+                        _('Cannot download news as no internet connection '
+                            'is active'))
+                d.setModal(False)
+                d.show()
+            return False
+        self.internet_connection_failed = False
         doit = urn not in self.download_queue
         self.lock.unlock()
         if doit:
             self.do_download(urn)
+        return True
 
     def check(self):
         recipes = self.recipe_model.get_to_be_downloaded_recipes()
