@@ -6,7 +6,7 @@ from functools import partial
 from threading import Thread
 
 from PyQt4.Qt import QApplication, Qt, QIcon, QTimer, SIGNAL, QByteArray, \
-                     QDesktopServices, QDoubleSpinBox, QLabel, QTextBrowser, \
+                     QDoubleSpinBox, QLabel, QTextBrowser, \
                      QPainter, QBrush, QColor, QStandardItemModel, QPalette, \
                      QStandardItem, QUrl, QRegExpValidator, QRegExp, QLineEdit, \
                      QToolButton, QMenu, QInputDialog, QAction, QKeySequence
@@ -17,10 +17,10 @@ from calibre.gui2.viewer.bookmarkmanager import BookmarkManager
 from calibre.gui2.widgets import ProgressIndicator
 from calibre.gui2.main_window import MainWindow
 from calibre.gui2 import Application, ORG_NAME, APP_UID, choose_files, \
-                         info_dialog, error_dialog
+                         info_dialog, error_dialog, open_url
 from calibre.ebooks.oeb.iterator import EbookIterator
 from calibre.ebooks import DRMError
-from calibre.constants import islinux
+from calibre.constants import islinux, isfreebsd
 from calibre.utils.config import Config, StringConfig, dynamic
 from calibre.gui2.search_box import SearchBox2
 from calibre.ebooks.metadata import MetaInformation
@@ -194,6 +194,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.tool_bar2.insertSeparator(self.action_find_next)
         self.setFocusPolicy(Qt.StrongFocus)
         self.search = SearchBox2(self)
+        self.search.setMinimumContentsLength(20)
         self.search.initialize('viewer_search_history')
         self.search.setToolTip(_('Search for text in book'))
         self.search.setMinimumWidth(200)
@@ -228,7 +229,11 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.connect(self.action_previous_page, SIGNAL('triggered(bool)'),
                      lambda x:self.view.previous_page())
         self.connect(self.action_find_next, SIGNAL('triggered(bool)'),
-                     lambda x:self.find(unicode(self.search.text()), True, repeat=True))
+                     lambda x:self.find(self.search.smart_text, repeat=True))
+        self.connect(self.action_find_previous, SIGNAL('triggered(bool)'),
+                     lambda x:self.find(self.search.smart_text,
+                         repeat=True, backwards=True))
+
         self.connect(self.action_full_screen, SIGNAL('triggered(bool)'),
                      self.toggle_fullscreen)
         self.action_full_screen.setShortcuts([Qt.Key_F11, Qt.CTRL+Qt.SHIFT+Qt.Key_F])
@@ -239,7 +244,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.pos.editingFinished.connect(self.goto_page_num)
         self.connect(self.vertical_scrollbar, SIGNAL('valueChanged(int)'),
                      lambda x: self.goto_page(x/100.))
-        self.connect(self.search, SIGNAL('search(PyQt_PyObject, PyQt_PyObject)'), self.find)
+        self.search.search.connect(self.find)
         self.connect(self.toc, SIGNAL('clicked(QModelIndex)'), self.toc_clicked)
         self.connect(self.reference, SIGNAL('goto(PyQt_PyObject)'), self.goto)
 
@@ -419,13 +424,15 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
             self.set_bookmarks(self.iterator.bookmarks)
 
 
-    def find(self, text, refinement, repeat=False):
+    def find(self, text, repeat=False, backwards=False):
         if not text:
+            self.view.search('')
             return self.search.search_done(False)
         if self.view.search(text):
             self.scrolled(self.view.scroll_fraction)
             return self.search.search_done(True)
-        index = self.iterator.search(text, self.current_index)
+        index = self.iterator.search(text, self.current_index,
+                backwards=backwards)
         if index is None:
             if self.current_index > 0:
                 index = self.iterator.search(text, 0)
@@ -443,10 +450,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
             self.scrolled(self.view.scroll_fraction)
 
     def keyPressEvent(self, event):
-        if event.key() == Qt.Key_F3:
-            text = unicode(self.search.text())
-            self.find(text, True, repeat=True)
-        elif event.key() == Qt.Key_Slash:
+        if event.key() == Qt.Key_Slash:
             self.search.setFocus(Qt.OtherFocusReason)
         else:
             return MainWindow.keyPressEvent(self, event)
@@ -468,7 +472,7 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
             elif frag:
                 self.view.scroll_to(frag)
         else:
-            QDesktopServices.openUrl(url)
+            open_url(url)
 
     def load_started(self):
         self.open_progress_indicator(_('Loading flow...'))
@@ -686,7 +690,7 @@ View an ebook.
 def main(args=sys.argv):
     parser = option_parser()
     opts, args = parser.parse_args(args)
-    pid = os.fork() if False and islinux else -1
+    pid = os.fork() if False and (islinux or isfreebsd) else -1
     if pid <= 0:
         app = Application(args)
         app.setWindowIcon(QIcon(I('viewer.svg')))
