@@ -4,25 +4,27 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 Miscellaneous widgets used in the GUI
 '''
 import re, os, traceback
+
 from PyQt4.Qt import QListView, QIcon, QFont, QLabel, QListWidget, \
                         QListWidgetItem, QTextCharFormat, QApplication, \
                         QSyntaxHighlighter, QCursor, QColor, QWidget, \
-                        QPixmap, QPalette, QTimer, QDialog, \
-                        QAbstractListModel, QVariant, Qt, SIGNAL, \
-                        QRegExp, QSettings, QSize, QModelIndex, \
+                        QPixmap, QPalette, QSplitterHandle, QToolButton, \
+                        QAbstractListModel, QVariant, Qt, SIGNAL, pyqtSignal, \
+                        QRegExp, QSettings, QSize, QModelIndex, QSplitter, \
                         QAbstractButton, QPainter, QLineEdit, QComboBox, \
-                        QMenu, QStringListModel, QCompleter, QStringList
+                        QMenu, QStringListModel, QCompleter, QStringList, \
+                        QTimer, QRect
+
+from calibre.gui2 import NONE, error_dialog, pixmap_to_data, gprefs
 
-from calibre.gui2 import human_readable, NONE, TableView, \
-                         qstring_to_unicode, error_dialog, pixmap_to_data
-from calibre.gui2.dialogs.job_view_ui import Ui_Dialog
 from calibre.gui2.filename_pattern_ui import Ui_Form
-from calibre import fit_image
+from calibre import fit_image, human_readable
 from calibre.utils.fonts import fontconfig
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.ebooks.metadata.meta import metadata_from_filename
 from calibre.utils.config import prefs, XMLConfig
 from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndicator
+from calibre.constants import filesystem_encoding
 
 history = XMLConfig('history')
 
@@ -36,12 +38,16 @@ class ProgressIndicator(QWidget):
         self.status.setWordWrap(True)
         self.status.setAlignment(Qt.AlignHCenter|Qt.AlignTop)
         self.setVisible(False)
+        self.pos = None
 
     def start(self, msg=''):
         view = self.parent()
         pwidth, pheight = view.size().width(), view.size().height()
         self.resize(pwidth, min(pheight, 250))
-        self.move(0, (pheight-self.size().height())/2.)
+        if self.pos is None:
+            self.move(0, (pheight-self.size().height())/2.)
+        else:
+            self.move(self.pos[0], self.pos[1])
         self.pi.resize(self.pi.sizeHint())
         self.pi.move(int((self.size().width()-self.pi.size().width())/2.), 0)
         self.status.resize(self.size().width(), self.size().height()-self.pi.size().height()-10)
@@ -71,7 +77,7 @@ class FilenamePattern(QWidget, Ui_Form):
             error_dialog(self, _('Invalid regular expression'),
                          _('Invalid regular expression: %s')%err).exec_()
             return
-        mi = metadata_from_filename(qstring_to_unicode(self.filename.text()), pat)
+        mi = metadata_from_filename(unicode(self.filename.text()), pat)
         if mi.title:
             self.title.setText(mi.title)
         else:
@@ -95,7 +101,7 @@ class FilenamePattern(QWidget, Ui_Form):
 
 
     def pattern(self):
-        pat = qstring_to_unicode(self.re.text())
+        pat = unicode(self.re.text())
         return re.compile(pat)
 
     def commit(self):
@@ -144,10 +150,15 @@ class FormatList(QListWidget):
             return QListWidget.keyPressEvent(self, event)
 
 
-class ImageView(QLabel):
+class ImageView(QWidget):
 
-    MAX_WIDTH  = 400
-    MAX_HEIGHT = 300
+    def __init__(self, parent=None):
+        QWidget.__init__(self, parent)
+        self._pixmap = QPixmap(self)
+        self.setMinimumSize(QSize(150, 200))
+        self.setAcceptDrops(True)
+
+    # Drag 'n drop {{{
     DROPABBLE_EXTENSIONS = IMAGE_EXTENSIONS
 
     @classmethod
@@ -157,7 +168,7 @@ class ImageView(QLabel):
         and represent files with extensions.
         '''
         if event.mimeData().hasFormat('text/uri-list'):
-            urls = [qstring_to_unicode(u.toLocalFile()) for u in event.mimeData().urls()]
+            urls = [unicode(u.toLocalFile()) for u in event.mimeData().urls()]
             urls = [u for u in urls if os.path.splitext(u)[1] and os.access(u, os.R_OK)]
             return [u for u in urls if os.path.splitext(u)[1][1:].lower() in cls.DROPABBLE_EXTENSIONS]
 
@@ -184,13 +195,45 @@ class ImageView(QLabel):
 
     def dragMoveEvent(self, event):
         event.acceptProposedAction()
+    # }}}
 
     def setPixmap(self, pixmap):
-        QLabel.setPixmap(self, pixmap)
-        width, height = fit_image(pixmap.width(), pixmap.height(), self.MAX_WIDTH, self.MAX_HEIGHT)[1:]
-        self.setMaximumWidth(width)
-        self.setMaximumHeight(height)
+        if not isinstance(pixmap, QPixmap):
+            raise TypeError('Must use a QPixmap')
+        self._pixmap = pixmap
+        self.updateGeometry()
+        self.update()
 
+    def pixmap(self):
+        return self._pixmap
+
+    def sizeHint(self):
+        if self._pixmap.isNull():
+            return self.minimumSize()
+        return self._pixmap.size()
+
+    def paintEvent(self, event):
+        QWidget.paintEvent(self, event)
+        pmap = self._pixmap
+        if pmap.isNull():
+            return
+        w, h = pmap.width(), pmap.height()
+        cw, ch = self.rect().width(), self.rect().height()
+        scaled, nw, nh = fit_image(w, h, cw, ch)
+        if scaled:
+            pmap = pmap.scaled(nw, nh, Qt.IgnoreAspectRatio,
+                    Qt.SmoothTransformation)
+        w, h = pmap.width(), pmap.height()
+        x = int(abs(cw - w)/2.)
+        y = int(abs(ch - h)/2.)
+        target = QRect(x, y, w, h)
+        p = QPainter(self)
+        p.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
+        p.drawPixmap(target, pmap)
+        p.end()
+
+
+    # Clipboard copy/paste # {{{
     def contextMenuEvent(self, ev):
         cm = QMenu(self)
         copy = cm.addAction(_('Copy Image'))
@@ -213,6 +256,7 @@ class ImageView(QLabel):
             self.setPixmap(pmap)
             self.emit(SIGNAL('cover_changed(PyQt_PyObject)'),
                     pixmap_to_data(pmap))
+    # }}}
 
 
 class LocationModel(QAbstractListModel):
@@ -230,13 +274,22 @@ class LocationModel(QAbstractListModel):
         self.free = [-1, -1, -1]
         self.count = 0
         self.highlight_row = 0
+        self.library_tooltip = _('Click to see the books available on your computer')
         self.tooltips = [
-                         _('Click to see the books available on your computer'),
+                         self.library_tooltip,
                          _('Click to see the books in the main memory of your reader'),
                          _('Click to see the books on storage card A in your reader'),
                          _('Click to see the books on storage card B in your reader')
                          ]
 
+    def database_changed(self, db):
+        lp = db.library_path
+        if not isinstance(lp, unicode):
+            lp = lp.decode(filesystem_encoding, 'replace')
+        self.tooltips[0] = self.library_tooltip + '\n\n' + \
+                _('Books located at') + ' ' + lp
+        self.dataChanged.emit(self.index(0), self.index(0))
+
     def rowCount(self, *args):
         return 1 + len([i for i in self.free if i >= 0])
 
@@ -389,41 +442,6 @@ class EjectButton(QAbstractButton):
         painter.drawPixmap(0, 0, image)
 
 
-class DetailView(QDialog, Ui_Dialog):
-
-    def __init__(self, parent, job):
-        QDialog.__init__(self, parent)
-        self.setupUi(self)
-        self.setWindowTitle(job.description)
-        self.job = job
-        self.next_pos = 0
-        self.update()
-        self.timer = QTimer(self)
-        self.connect(self.timer, SIGNAL('timeout()'), self.update)
-        self.timer.start(1000)
-
-
-    def update(self):
-        f = self.job.log_file
-        f.seek(self.next_pos)
-        more = f.read()
-        self.next_pos = f.tell()
-        if more:
-            self.log.appendPlainText(more.decode('utf-8', 'replace'))
-
-
-class JobsView(TableView):
-
-    def __init__(self, parent):
-        TableView.__init__(self, parent)
-        self.connect(self, SIGNAL('doubleClicked(QModelIndex)'), self.show_details)
-
-    def show_details(self, index):
-        row = index.row()
-        job = self.model().row_to_job(row)
-        d = DetailView(self, job)
-        d.exec_()
-        d.timer.stop()
 
 
 class FontFamilyModel(QAbstractListModel):
@@ -520,7 +538,7 @@ class BasicList(QListWidget):
 class LineEditECM(object):
 
     '''
-    Extend the contenxt menu of a QLineEdit to include more actions.
+    Extend the context menu of a QLineEdit to include more actions.
     '''
 
     def contextMenuEvent(self, event):
@@ -551,7 +569,8 @@ class LineEditECM(object):
         self.setText(unicode(self.text()).swapcase())
 
     def title_case(self):
-        self.setText(unicode(self.text()).title())
+        from calibre.utils.titlecase import titlecase
+        self.setText(titlecase(unicode(self.text())))
 
 
 class EnLineEdit(LineEditECM, QLineEdit):
@@ -619,13 +638,13 @@ class TagsLineEdit(EnLineEdit):
         self.completer.update_tags_cache(tags)
 
     def text_changed(self, text):
-        all_text = qstring_to_unicode(text)
+        all_text = unicode(text)
         text = all_text[:self.cursorPosition()]
         prefix = text.split(',')[-1].strip()
 
         text_tags = []
         for t in all_text.split(self.separator):
-            t1 = qstring_to_unicode(t).strip()
+            t1 = unicode(t).strip()
             if t1 != '':
                 text_tags.append(t)
         text_tags = list(set(text_tags))
@@ -635,8 +654,8 @@ class TagsLineEdit(EnLineEdit):
 
     def complete_text(self, text):
         cursor_pos = self.cursorPosition()
-        before_text = qstring_to_unicode(self.text())[:cursor_pos]
-        after_text = qstring_to_unicode(self.text())[cursor_pos:]
+        before_text = unicode(self.text())[:cursor_pos]
+        after_text = unicode(self.text())[cursor_pos:]
         prefix_len = len(before_text.split(',')[-1].strip())
         self.setText('%s%s%s %s' % (before_text[:cursor_pos - prefix_len],
             text, self.separator, after_text))
@@ -648,7 +667,7 @@ class EnComboBox(QComboBox):
     '''
     Enhanced QComboBox.
 
-    Includes an extended content menu.
+    Includes an extended context menu.
     '''
 
     def __init__(self, *args):
@@ -940,3 +959,213 @@ class PythonHighlighter(QSyntaxHighlighter):
         QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
         QSyntaxHighlighter.rehighlight(self)
         QApplication.restoreOverrideCursor()
+
+class SplitterHandle(QSplitterHandle):
+
+    double_clicked = pyqtSignal(object)
+
+    def __init__(self, orientation, splitter):
+        QSplitterHandle.__init__(self, orientation, splitter)
+        splitter.splitterMoved.connect(self.splitter_moved,
+                type=Qt.QueuedConnection)
+        self.double_clicked.connect(splitter.double_clicked,
+                type=Qt.QueuedConnection)
+        self.highlight = False
+        self.setToolTip(_('Drag to resize')+' '+splitter.label)
+
+    def splitter_moved(self, *args):
+        oh = self.highlight
+        self.highlight = 0 in self.splitter().sizes()
+        if oh != self.highlight:
+            self.update()
+
+    def paintEvent(self, ev):
+        QSplitterHandle.paintEvent(self, ev)
+        if self.highlight:
+            painter = QPainter(self)
+            painter.setClipRect(ev.rect())
+            painter.fillRect(self.rect(), Qt.yellow)
+
+    def mouseDoubleClickEvent(self, ev):
+        self.double_clicked.emit(self)
+
+class LayoutButton(QToolButton):
+
+    def __init__(self, icon, text, splitter, parent=None):
+        QToolButton.__init__(self, parent)
+        self.label = text
+        self.setIcon(QIcon(icon))
+        self.setCheckable(True)
+
+        self.splitter = splitter
+        splitter.state_changed.connect(self.update_state)
+        self.setCursor(Qt.PointingHandCursor)
+
+    def set_state_to_show(self, *args):
+        self.setChecked(False)
+        label =_('Show')
+        self.setText(label + ' ' + self.label)
+        self.setToolTip(self.text())
+
+    def set_state_to_hide(self, *args):
+        self.setChecked(True)
+        label = _('Hide')
+        self.setText(label + ' ' + self.label)
+        self.setToolTip(self.text())
+
+    def update_state(self, *args):
+        if self.splitter.is_side_index_hidden:
+            self.set_state_to_show()
+        else:
+            self.set_state_to_hide()
+
+class Splitter(QSplitter):
+
+    state_changed = pyqtSignal(object)
+
+    def __init__(self, name, label, icon, initial_show=True,
+            initial_side_size=120, connect_button=True,
+            orientation=Qt.Horizontal, side_index=0, parent=None):
+        QSplitter.__init__(self, parent)
+        self.resize_timer = QTimer(self)
+        self.resize_timer.setSingleShot(True)
+        self.desired_side_size = initial_side_size
+        self.desired_show = initial_show
+        self.resize_timer.setInterval(5)
+        self.resize_timer.timeout.connect(self.do_resize)
+        self.setOrientation(orientation)
+        self.side_index = side_index
+        self._name = name
+        self.label = label
+        self.initial_side_size = initial_side_size
+        self.initial_show = initial_show
+        self.splitterMoved.connect(self.splitter_moved, type=Qt.QueuedConnection)
+        self.button = LayoutButton(icon, label, self)
+        if connect_button:
+            self.button.clicked.connect(self.double_clicked)
+
+    def createHandle(self):
+        return SplitterHandle(self.orientation(), self)
+
+    def initialize(self):
+        for i in range(self.count()):
+            h = self.handle(i)
+            if h is not None:
+                h.splitter_moved()
+        self.state_changed.emit(not self.is_side_index_hidden)
+
+    def splitter_moved(self, *args):
+        self.desired_side_size = self.side_index_size
+        self.state_changed.emit(not self.is_side_index_hidden)
+
+    @property
+    def is_side_index_hidden(self):
+        sizes = list(self.sizes())
+        return sizes[self.side_index] == 0
+
+    @property
+    def save_name(self):
+        ori = 'horizontal' if self.orientation() == Qt.Horizontal \
+                else 'vertical'
+        return self._name + '_' + ori
+
+    def print_sizes(self):
+        if self.count() > 1:
+            print self.save_name, 'side:', self.side_index_size, 'other:',
+            print list(self.sizes())[self.other_index]
+
+    @dynamic_property
+    def side_index_size(self):
+        def fget(self):
+            if self.count() < 2: return 0
+            return self.sizes()[self.side_index]
+
+        def fset(self, val):
+            if self.count() < 2: return
+            if val == 0 and not self.is_side_index_hidden:
+                self.save_state()
+            sizes = list(self.sizes())
+            for i in range(len(sizes)):
+                sizes[i] = val if i == self.side_index else 10
+            self.setSizes(sizes)
+            total = sum(self.sizes())
+            sizes = list(self.sizes())
+            for i in range(len(sizes)):
+                sizes[i] = val if i == self.side_index else total-val
+            self.setSizes(sizes)
+            self.initialize()
+
+        return property(fget=fget, fset=fset)
+
+    def do_resize(self, *args):
+        orig = self.desired_side_size
+        QSplitter.resizeEvent(self, self._resize_ev)
+        if orig > 20 and self.desired_show:
+            c = 0
+            while abs(self.side_index_size - orig) > 10 and c < 5:
+                self.apply_state(self.get_state(), save_desired=False)
+                c += 1
+
+    def resizeEvent(self, ev):
+        if self.resize_timer.isActive():
+            self.resize_timer.stop()
+        self._resize_ev = ev
+        self.resize_timer.start()
+
+    def get_state(self):
+        if self.count() < 2: return (False, 200)
+        return (self.desired_show, self.desired_side_size)
+
+    def apply_state(self, state, save_desired=True):
+        if state[0]:
+            self.side_index_size = state[1]
+            if save_desired:
+                self.desired_side_size = self.side_index_size
+        else:
+            self.side_index_size = 0
+        self.desired_show = state[0]
+
+    def default_state(self):
+        return (self.initial_show, self.initial_side_size)
+
+    # Public API {{{
+
+    def save_state(self):
+        if self.count() > 1:
+            gprefs[self.save_name+'_state'] = self.get_state()
+
+    @property
+    def other_index(self):
+        return (self.side_index+1)%2
+
+    def restore_state(self):
+        if self.count() > 1:
+            state = gprefs.get(self.save_name+'_state',
+                    self.default_state())
+            self.apply_state(state, save_desired=False)
+            self.desired_side_size = state[1]
+
+    def toggle_side_pane(self, hide=None):
+        if hide is None:
+            action = 'show' if self.is_side_index_hidden else 'hide'
+        else:
+            action = 'hide' if hide else 'show'
+        getattr(self, action+'_side_pane')()
+
+    def show_side_pane(self):
+        if self.count() < 2 or not self.is_side_index_hidden:
+            return
+        if self.desired_side_size == 0:
+            self.desired_side_size = self.initial_side_size
+        self.apply_state((True, self.desired_side_size))
+
+    def hide_side_pane(self):
+        if self.count() < 2 or self.is_side_index_hidden:
+            return
+        self.apply_state((False, self.desired_side_size))
+
+    def double_clicked(self, *args):
+        self.toggle_side_pane()
+
+    # }}}
+
