@@ -78,20 +78,38 @@ class KindleDX(Kindle):
     name = 'Kindle DX'
     id = 'kindledx'
 
-class Sony500(Device):
+class Sony505(Device):
 
     output_profile = 'sony'
-    name = 'SONY PRS 500'
-    output_format = 'LRF'
-    manufacturer = 'SONY'
-    id = 'prs500'
-
-class Sony505(Sony500):
-
+    name = 'All other SONY devices'
     output_format = 'EPUB'
-    name = 'SONY Reader 6" and Touch Edition'
+    manufacturer = 'SONY'
     id = 'prs505'
 
+class Kobo(Device):
+    name = 'Kobo Reader'
+    manufacturer = 'Kobo'
+    output_profile = 'kobo'
+    output_format = 'EPUB'
+    id = 'kobo'
+
+class Booq(Device):
+    name = 'Booq Reader'
+    manufacturer = 'Booq'
+    output_profile = 'sony'
+    output_format = 'EPUB'
+    id = 'booq'
+
+class TheBook(Device):
+    name = 'The Book'
+    manufacturer = 'Augen'
+    output_profile = 'sony'
+    output_format = 'EPUB'
+    id = 'thebook'
+
+class Avant(Booq):
+    name = 'Booq Avant'
+
 class Sony300(Sony505):
 
     name = 'SONY Reader Pocket Edition'
@@ -125,9 +143,23 @@ class CybookOpus(CybookG3):
     output_profile = 'cybook_opus'
     id = 'cybook_opus'
 
+class PocketBook360(CybookOpus):
+
+    manufacturer = 'PocketBook'
+    name = 'PocketBook 360'
+    id = 'pocketbook360'
+    output_profile = 'cybook_opus'
+
+class PocketBook(CybookG3):
+
+    manufacturer = 'PocketBook'
+    name = 'PocketBook 301/302'
+    id = 'pocketbook'
+    output_profile = 'cybookg3'
+
 class iPhone(Device):
 
-    name = 'iPhone/iTouch + Stanza'
+    name = 'iPad or iPhone/iTouch + Stanza'
     output_format = 'EPUB'
     manufacturer = 'Apple'
     id = 'iphone'
@@ -299,9 +331,14 @@ class StanzaPage(QWizardPage, StanzaUI):
         return FinishPage.ID
 
     def commit(self):
+        from calibre.ebooks.conversion.config import load_defaults, save_defaults
+        recs = load_defaults('comic_input')
+        recs['dont_grayscale'] = True
+        save_defaults('comic_input', recs)
+
         p = self.set_port()
         if p is not None:
-            from calibre.library import server_config
+            from calibre.library.server import server_config
             c = server_config()
             c.set('port', p)
 
