@@ -6,7 +6,9 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 import sys, os, shutil, cPickle, textwrap, stat
 from subprocess import check_call
 
-from calibre import  __appname__, prints
+from calibre import  __appname__, prints, guess_type
+from calibre.constants import islinux, isfreebsd
+from calibre.customize.ui import all_input_formats
 
 
 entry_points = {
@@ -16,7 +18,7 @@ entry_points = {
              'ebook-convert      = calibre.ebooks.conversion.cli:main',
              'markdown-calibre   = calibre.ebooks.markdown.markdown:main',
              'web2disk           = calibre.web.fetch.simple:main',
-             'calibre-server     = calibre.library.server:main',
+             'calibre-server     = calibre.library.server.main:main',
              'lrf2lrs            = calibre.ebooks.lrf.lrfparser:main',
              'lrs2lrf            = calibre.ebooks.lrf.lrs.convert_from:main',
              'librarything       = calibre.ebooks.metadata.library_thing:main',
@@ -128,20 +130,22 @@ class PostInstall:
         self.icon_resources = []
         self.menu_resources = []
         self.mime_resources = []
-        self.setup_completion()
-        self.setup_udev_rules()
+        if islinux:
+            self.setup_completion()
         self.install_man_pages()
-        self.setup_desktop_integration()
+        if islinux:
+            self.setup_desktop_integration()
         self.create_uninstaller()
 
         from calibre.utils.config import config_dir
         if os.path.exists(config_dir):
             os.chdir(config_dir)
-            for f in os.listdir('.'):
-                if os.stat(f).st_uid == 0:
-                    os.rmdir(f) if os.path.isdir(f) else os.unlink(f)
-            if os.stat(config_dir).st_uid == 0:
-                os.rmdir(config_dir)
+            if islinux:
+                for f in os.listdir('.'):
+                    if os.stat(f).st_uid == 0:
+                        os.rmdir(f) if os.path.isdir(f) else os.unlink(f)
+                if os.stat(config_dir).st_uid == 0:
+                    os.rmdir(config_dir)
 
         if warn is None and self.warnings:
             self.info('There were %d warnings'%len(self.warnings))
@@ -281,44 +285,13 @@ class PostInstall:
                 raise
             self.task_failed('Setting up completion failed')
 
-    def setup_udev_rules(self):
-        self.info('Trying to setup udev rules...')
-        try:
-            group_file = os.path.join(self.opts.staging_etc, 'group')
-            if not os.path.exists(group_file):
-                group_file = '/etc/group'
-            groups = open(group_file, 'rb').read()
-            group = 'plugdev' if 'plugdev' in groups else 'usb'
-            old_udev = '/etc/udev/rules.d/95-calibre.rules'
-            if not os.path.exists(old_udev):
-                old_udev = os.path.join(self.opts.staging_etc, 'udev/rules.d/95-calibre.rules')
-            if os.path.exists(old_udev):
-                try:
-                    os.remove(old_udev)
-                except:
-                    self.warn('Old udev rules found, please delete manually:',
-                            old_udev)
-            if self.opts.staging_root == '/usr':
-                base = '/lib'
-            else:
-                base = os.path.join(self.opts.staging_root, 'lib')
-            base = os.path.join(base, 'udev', 'rules.d')
-            if not os.path.exists(base):
-                os.makedirs(base)
-            with open(os.path.join(base, '95-calibre.rules'), 'wb') as udev:
-                self.manifest.append(udev.name)
-                udev.write('''# Sony Reader PRS-500\n'''
-                        '''BUS=="usb", SYSFS{idProduct}=="029b", SYSFS{idVendor}=="054c", MODE="660", GROUP="%s"\n'''%(group,)
-                        )
-        except:
-            if self.opts.fatal_errors:
-                raise
-            self.task_failed('Setting up udev rules failed')
-
     def install_man_pages(self):
         try:
             from calibre.utils.help2man import create_man_page
-            manpath = os.path.join(self.opts.staging_sharedir, 'man/man1')
+            if isfreebsd:
+                manpath = os.path.join(self.opts.staging_root, 'man/man1')
+            else:
+                manpath = os.path.join(self.opts.staging_sharedir, 'man/man1')
             if not os.path.exists(manpath):
                 os.makedirs(manpath)
             self.info('Installing MAN pages...')
@@ -331,7 +304,10 @@ class PostInstall:
                     continue
                 parser = parser()
                 raw = create_man_page(prog, parser)
-                manfile = os.path.join(manpath, prog+'.1'+__appname__+'.bz2')
+                if isfreebsd:
+                    manfile = os.path.join(manpath, prog+'.1')
+                else:
+                    manfile = os.path.join(manpath, prog+'.1'+__appname__+'.bz2')
                 self.info('\tInstalling MAN page for', prog)
                 open(manfile, 'wb').write(raw)
                 self.manifest.append(manfile)
@@ -365,14 +341,25 @@ class PostInstall:
                 check_call('xdg-icon-resource install --size 128 calibre-viewer.png calibre-viewer', shell=True)
                 self.icon_resources.append(('apps', 'calibre-viewer', '128'))
 
+                mimetypes = set([])
+                for x in all_input_formats():
+                    mt = guess_type('dummy.'+x)[0]
+                    if mt and 'chemical' not in mt:
+                        mimetypes.add(mt)
+
+                def write_mimetypes(f):
+                    f.write('MimeType=%s;\n'%';'.join(mimetypes))
+
                 f = open('calibre-lrfviewer.desktop', 'wb')
                 f.write(VIEWER)
                 f.close()
                 f = open('calibre-ebook-viewer.desktop', 'wb')
                 f.write(EVIEWER)
+                write_mimetypes(f)
                 f.close()
                 f = open('calibre-gui.desktop', 'wb')
                 f.write(GUI)
+                write_mimetypes(f)
                 f.close()
                 des = ('calibre-gui.desktop', 'calibre-lrfviewer.desktop',
                         'calibre-ebook-viewer.desktop')
@@ -424,7 +411,8 @@ def options(option_parser):
 def opts_and_words(name, op, words):
     opts  = '|'.join(options(op))
     words = '|'.join([w.replace("'", "\\'") for w in words])
-    return ('_'+name+'()'+\
+    fname = name.replace('-', '_')
+    return ('_'+fname+'()'+\
 '''
 {
     local cur opts
@@ -448,14 +436,15 @@ def opts_and_words(name, op, words):
     esac
 
 }
-complete -F _'''%(opts, words) + name + ' ' + name +"\n\n").encode('utf-8')
+complete -F _'''%(opts, words) + fname + ' ' + name +"\n\n").encode('utf-8')
 
 
 def opts_and_exts(name, op, exts):
     opts = ' '.join(options(op))
     exts.extend([i.upper() for i in exts])
     exts='|'.join(exts)
-    return '_'+name+'()'+\
+    fname = name.replace('-', '_')
+    return '_'+fname+'()'+\
 '''
 {
     local cur prev opts
@@ -488,7 +477,7 @@ def opts_and_exts(name, op, exts):
     esac
 
 }
-complete -o filenames -F _'''%(opts,exts) + name + ' ' + name +"\n\n"
+complete -o filenames -F _'''%(opts,exts) + fname + ' ' + name +"\n\n"
 
 
 VIEWER = '''\
@@ -511,11 +500,10 @@ Version=1.0
 Type=Application
 Name=E-book Viewer
 GenericName=Viewer for E-books
-Comment=Viewer for E-books
+Comment=Viewer for E-books in all the major formats
 TryExec=ebook-viewer
 Exec=ebook-viewer %F
 Icon=calibre-viewer
-MimeType=application/epub+zip;
 Categories=Graphics;Viewer;
 '''
 
@@ -526,7 +514,7 @@ Version=1.0
 Type=Application
 Name=calibre
 GenericName=E-book library management
-Comment=E-book library management
+Comment=E-book library management: Convert, view, share, catalogue all your e-books
 TryExec=calibre
 Exec=calibre
 Icon=calibre-gui
@@ -551,17 +539,20 @@ MIME = '''\
 </mime-info>
 '''
 
-def render_svg(image, dest):
+def render_svg(image, dest, width=128, height=128):
     from PyQt4.QtGui import QPainter, QImage
     from PyQt4.QtSvg import QSvgRenderer
-    svg = QSvgRenderer(image.readAll())
+    image = image.readAll() if hasattr(image, 'readAll') else image
+    svg = QSvgRenderer(image)
     painter = QPainter()
-    image = QImage(128,128,QImage.Format_ARGB32_Premultiplied)
+    image = QImage(width, height, QImage.Format_ARGB32)
     painter.begin(image)
     painter.setRenderHints(QPainter.Antialiasing|QPainter.TextAntialiasing|QPainter.SmoothPixmapTransform|QPainter.HighQualityAntialiasing)
     painter.setCompositionMode(QPainter.CompositionMode_SourceOver)
     svg.render(painter)
     painter.end()
+    if dest is None:
+        return image
     image.save(dest)
 
 def main():
