@@ -6,7 +6,7 @@ __docformat__ = 'restructuredtext en'
 '''
 Manage application-wide preferences.
 '''
-import os, re, cPickle, textwrap, traceback, plistlib, json
+import os, re, cPickle, textwrap, traceback, plistlib, json, base64
 from copy import deepcopy
 from functools import partial
 from optparse import OptionParser as _OptionParser
@@ -636,11 +636,31 @@ class JSONConfig(XMLConfig):
 
     EXTENSION = '.json'
 
+    def to_json(self, obj):
+        if isinstance(obj, bytearray):
+            return {'__class__': 'bytearray',
+                    '__value__': base64.standard_b64encode(bytes(obj))}
+        raise TypeError(repr(obj) + ' is not JSON serializable')
+
+    def from_json(self, obj):
+        if '__class__' in obj:
+            if obj['__class__'] == 'bytearray':
+                return bytearray(base64.standard_b64decode(obj['__value__']))
+        return obj
+
     def raw_to_object(self, raw):
-        return json.loads(raw.decode('utf-8'))
+        return json.loads(raw.decode('utf-8'), object_hook=self.from_json)
 
     def to_raw(self):
-        return json.dumps(self, indent=2)
+        return json.dumps(self, indent=2, default=self.to_json)
+
+    def __getitem__(self, key):
+        return dict.__getitem__(self, key)
+
+    def __setitem__(self, key, val):
+        dict.__setitem__(self, key, val)
+        self.commit()
+
 
 
 def _prefs():
@@ -670,11 +690,24 @@ def _prefs():
               help=_('The priority of worker processes'))
     c.add_opt('swap_author_names', default=False,
             help=_('Swap author first and last names when reading metadata'))
+    c.add_opt('add_formats_to_existing', default=False,
+            help=_('Add new formats to existing book records'))
+    c.add_opt('installation_uuid', default=None, help='Installation UUID')
+
+    # these are here instead of the gui preferences because calibredb and
+    # calibre server can execute searches
+    c.add_opt('saved_searches', default={}, help=_('List of named saved searches'))
+    c.add_opt('user_categories', default={}, help=_('User-created tag browser categories'))
+    c.add_opt('preserve_user_collections', default=True,
+        help=_('Preserve all collections even if not in library metadata.'))
 
     c.add_opt('migrated', default=False, help='For Internal use. Don\'t modify.')
     return c
 
 prefs = ConfigProxy(_prefs())
+if prefs['installation_uuid'] is None:
+    import uuid
+    prefs['installation_uuid'] = str(uuid.uuid4())
 
 # Read tweaks
 def read_tweaks():
