@@ -6,63 +6,97 @@ __docformat__ = 'restructuredtext en'
 
 '''
 A parser for search queries with a syntax very similar to that used by
-the Google search engine. 
+the Google search engine.
 
-For details on the search query syntax see :class:`SearchQueryParser`. 
-To use the parser, subclass :class:`SearchQueryParser` and implement the 
-methods :method:`SearchQueryParser.universal_set` and 
+For details on the search query syntax see :class:`SearchQueryParser`.
+To use the parser, subclass :class:`SearchQueryParser` and implement the
+methods :method:`SearchQueryParser.universal_set` and
 :method:`SearchQueryParser.get_matches`. See for example :class:`Tester`.
 
-If this module is run, it will perform a series of unit tests. 
+If this module is run, it will perform a series of unit tests.
 '''
 
 import sys, string, operator
 
 from calibre.utils.pyparsing import Keyword, Group, Forward, CharsNotIn, Suppress, \
-                      OneOrMore, oneOf, CaselessLiteral, Optional, NoMatch
+                      OneOrMore, oneOf, CaselessLiteral, Optional, NoMatch, ParseException
+from calibre.constants import preferred_encoding
+from calibre.utils.config import prefs
+
+'''
+This class manages access to the preference holding the saved search queries.
+It exists to ensure that unicode is used throughout, and also to permit
+adding other fields, such as whether the search is a 'favorite'
+'''
+class SavedSearchQueries(object):
+    queries = {}
+    opt_name = ''
+
+    def __init__(self, _opt_name):
+        self.opt_name = _opt_name;
+        self.queries = prefs[self.opt_name]
+
+    def force_unicode(self, x):
+        if not isinstance(x, unicode):
+            x = x.decode(preferred_encoding, 'replace')
+        return x
+
+    def add(self, name, value):
+        self.queries[self.force_unicode(name)] = self.force_unicode(value).strip()
+        prefs[self.opt_name] = self.queries
+
+    def lookup(self, name):
+        return self.queries.get(self.force_unicode(name), None)
+
+    def delete(self, name):
+        self.queries.pop(self.force_unicode(name), False)
+        prefs[self.opt_name] = self.queries
+
+    def rename(self, old_name, new_name):
+        self.queries[self.force_unicode(new_name)] = \
+                    self.queries.get(self.force_unicode(old_name), None)
+        self.queries.pop(self.force_unicode(old_name), False)
+        prefs[self.opt_name] = self.queries
+
+    def names(self):
+        return sorted(self.queries.keys(),
+                cmp=lambda x,y: cmp(x.lower(), y.lower()))
+
+'''
+Create a global instance of the saved searches. It is global so that the searches
+are common across all instances of the parser (devices, library, etc).
+'''
+saved_searches = SavedSearchQueries('saved_searches')
 
 
 class SearchQueryParser(object):
     '''
-    Parses a search query. 
-    
+    Parses a search query.
+
     A search query consists of tokens. The tokens can be combined using
-    the `or`, `and` and `not` operators as well as grouped using parentheses. 
+    the `or`, `and` and `not` operators as well as grouped using parentheses.
     When no operator is specified between two tokens, `and` is assumed.
-    
+
     Each token is a string of the form `location:query`. `location` is a string
-    from :member:`LOCATIONS`. It is optional. If it is omitted, it is assumed to 
-    be `all`. `query` is an arbitrary string that must not contain parentheses. 
+    from :member:`DEFAULT_LOCATIONS`. It is optional. If it is omitted, it is assumed to
+    be `all`. `query` is an arbitrary string that must not contain parentheses.
     If it contains whitespace, it should be quoted by enclosing it in `"` marks.
-    
+
     Examples::
-      
+
       * `Asimov` [search for the string "Asimov" in location `all`]
       * `comments:"This is a good book"` [search for "This is a good book" in `comments`]
       * `author:Asimov tag:unread` [search for books by Asimov that have been tagged as unread]
       * `author:Asimov or author:Hardy` [search for books by Asimov or Hardy]
-      * `(author:Asimov or author:Hardy) and not tag:read` [search for unread books by Asimov or Hardy] 
+      * `(author:Asimov or author:Hardy) and not tag:read` [search for unread books by Asimov or Hardy]
     '''
-    
-    LOCATIONS = [
-        'tag',
-        'title',
-        'author',
-        'publisher',
-        'series',
-        'rating',
-        'cover',
-        'comments',
-        'format',
-        'isbn',
-        'all',
-                 ]
-    
+
+
     @staticmethod
     def run_tests(parser, result, tests):
         failed = []
         for test in tests:
-            print '\tTesting:', test[0], 
+            print '\tTesting:', test[0],
             res = parser.parseString(test[0])
             if list(res.get(result, None)) == test[1]:
                 print 'OK'
@@ -70,14 +104,14 @@ class SearchQueryParser(object):
                 print 'FAILED:', 'Expected:', test[1], 'Got:', list(res.get(result, None))
                 failed.append(test[0])
         return failed
-    
-    def __init__(self, test=False):
+
+    def __init__(self, locations, test=False):
         self._tests_failed = False
         # Define a token
-        locations = map(lambda x : CaselessLiteral(x)+Suppress(':'),
-                        self.LOCATIONS)
+        standard_locations = map(lambda x : CaselessLiteral(x)+Suppress(':'),
+                locations)
         location = NoMatch()
-        for l in locations:
+        for l in standard_locations:
             location |= l
         location     = Optional(location, default='all')
         word_query   = CharsNotIn(string.whitespace + '()')
@@ -95,50 +129,63 @@ class SearchQueryParser(object):
                  ('title:"one two"',   ['title', 'one two']),
                 )
             )
-        
+
         Or = Forward()
-        
+
         Parenthesis = Group(
                         Suppress('(') + Or + Suppress(')')
                         ).setResultsName('parenthesis') | Token
-        
-         
+
+
         Not = Forward()
         Not << (Group(
             Suppress(Keyword("not", caseless=True)) + Not
         ).setResultsName("not") | Parenthesis)
-        
+
         And = Forward()
         And << (Group(
             Not + Suppress(Keyword("and", caseless=True)) + And
         ).setResultsName("and") | Group(
-            Not + OneOrMore(~oneOf("and or") + And)
+            Not + OneOrMore(~oneOf("and or", caseless=True) + And)
         ).setResultsName("and") | Not)
-        
+
         Or << (Group(
             And + Suppress(Keyword("or", caseless=True)) + Or
         ).setResultsName("or") | And)
-        
+
         if test:
             Or.validate()
             self._tests_failed = bool(failed)
-        
+
         self._parser = Or
         #self._parser.setDebug(True)
         #self.parse('(tolstoy)')
         self._parser.setDebug(False)
-        
-        
+
+
     def parse(self, query):
+        # empty the list of searches used for recursion testing
+        self.recurse_level = 0
+        self.searches_seen = set([])
+        return self._parse(query)
+
+    # this parse is used internally because it doesn't clear the
+    # recursive search test list. However, we permit seeing the
+    # same search a few times because the search might appear within
+    # another search.
+    def _parse(self, query):
+        self.recurse_level += 1
         res = self._parser.parseString(query)[0]
-        return self.evaluate(res)
-        
+        t = self.evaluate(res)
+        self.recurse_level -= 1
+        return t
+
     def method(self, group_name):
         return getattr(self, 'evaluate_'+group_name)
-    
+
     def evaluate(self, parse_result):
         return self.method(parse_result.getName())(parse_result)
-    
+
     def evaluate_and(self, argument):
         return self.evaluate(argument[0]).intersection(self.evaluate(argument[1]))
 
@@ -150,27 +197,40 @@ class SearchQueryParser(object):
 
     def evaluate_parenthesis(self, argument):
         return self.evaluate(argument[0])
-    
+
     def evaluate_token(self, argument):
-        return self.get_matches(argument[0], argument[1])
-        
+        location = argument[0]
+        query = argument[1]
+        if location.lower() == 'search':
+            if query.startswith('='):
+                query = query[1:]
+            try:
+                if query in self.searches_seen:
+                    raise ParseException(query, len(query), 'undefined saved search', self)
+                if self.recurse_level > 5:
+                    self.searches_seen.add(query)
+                return self._parse(saved_searches.lookup(query))
+            except: # convert all exceptions (e.g., missing key) to a parse error
+                raise ParseException(query, len(query), 'undefined saved search', self)
+        return self.get_matches(location, query)
+
     def get_matches(self, location, query):
         '''
-        Should return the set of matches for :param:'location` and :param:`query`. 
-        
-        :param:`location` is one of the items in :member:`SearchQueryParser.LOCATIONS`.
-        :param:`query` is a string literal.        
+        Should return the set of matches for :param:'location` and :param:`query`.
+
+        :param:`location` is one of the items in :member:`SearchQueryParser.DEFAULT_LOCATIONS`.
+        :param:`query` is a string literal.
         '''
         return set([])
-        
+
     def universal_set(self):
         '''
         Should return the set of all matches.
         '''
         return set([])
-    
+
 class Tester(SearchQueryParser):
-    
+
     texts = {
  1: [u'Eugenie Grandet', u'Honor\xe9 de Balzac', u'manybooks.net', u'lrf'],
  2: [u'Fanny Hill', u'John Cleland', u'manybooks.net', u'lrf'],
@@ -459,30 +519,30 @@ class Tester(SearchQueryParser):
        u'Washington Square Press',
        u'lrf,rar']
  }
-    
+
     tests = {
              'Dysfunction' : set([348]),
              'title:Dysfunction' : set([348]),
-             'title:Dysfunction or author:Laurie': set([348, 444]),
+             'title:Dysfunction OR author:Laurie': set([348, 444]),
              '(tag:txt or tag:pdf)': set([33, 258, 354, 305, 242, 51, 55, 56, 154]),
-             '(tag:txt or tag:pdf) and author:Tolstoy': set([55, 56]),
+             '(tag:txt OR tag:pdf) and author:Tolstoy': set([55, 56]),
              'Tolstoy txt': set([55, 56]),
              'Hamilton Amsterdam' : set([]),
              u'Beär' : set([91]),
              'dysfunc or tolstoy': set([348, 55, 56]),
-             'tag:txt and not tolstoy': set([33, 258, 354, 305, 242, 154]),
+             'tag:txt AND NOT tolstoy': set([33, 258, 354, 305, 242, 154]),
              'not tag:lrf' : set([305]),
              'london:thames': set([13]),
              'publisher:london:thames': set([13]),
              '"(1977)"': set([13]),
              }
     fields = {'title':0, 'author':1, 'publisher':2, 'tag':3}
-    
+
     _universal_set = set(texts.keys())
-    
+
     def universal_set(self):
         return self._universal_set
-    
+
     def get_matches(self, location, query):
         location = location.lower()
         if location in self.fields.keys():
@@ -491,19 +551,19 @@ class Tester(SearchQueryParser):
             getter = lambda y: ''.join(x if x else '' for x in y)
         else:
             getter = lambda x: ''
-        
+
         if not query:
-            return set([])    
+            return set([])
         query = query.lower()
         return set(key for key, val in self.texts.items() \
             if query and query in getattr(getter(val), 'lower', lambda : '')())
-            
-        
-    
+
+
+
     def run_tests(self):
         failed = []
         for query in self.tests.keys():
-            print 'Testing query:', query, 
+            print 'Testing query:', query,
             res = self.parse(query)
             if res != self.tests[query]:
                 print 'FAILED', 'Expected:', self.tests[query], 'Got:', res
@@ -511,7 +571,7 @@ class Tester(SearchQueryParser):
             else:
                 print 'OK'
         return failed
-                 
+
 
 def main(args=sys.argv):
     tester = Tester(test=True)
@@ -519,7 +579,7 @@ def main(args=sys.argv):
     if tester._tests_failed or failed:
         print '>>>>>>>>>>>>>> Tests Failed <<<<<<<<<<<<<<<'
         return 1
-    
+
     return 0
 
 if __name__ == '__main__':
