@@ -14,22 +14,27 @@ from contextlib import nested, closing
 
 
 from calibre import browser, __appname__, iswindows, \
-                    strftime, __version__, preferred_encoding
+                    strftime, preferred_encoding
 from calibre.ebooks.BeautifulSoup import BeautifulSoup, NavigableString, CData, Tag
 from calibre.ebooks.metadata.opf2 import OPFCreator
 from calibre import entity_to_unicode
 from calibre.web import Recipe
-from calibre.ebooks import render_html
 from calibre.ebooks.metadata.toc import TOC
 from calibre.ebooks.metadata import MetaInformation
 from calibre.web.feeds import feed_from_xml, templates, feeds_from_index, Feed
 from calibre.web.fetch.simple import option_parser as web2disk_option_parser
 from calibre.web.fetch.simple import RecursiveFetcher
+from calibre.utils.magick_draw import add_borders_to_image
 from calibre.utils.threadpool import WorkRequest, ThreadPool, NoResultsPending
-from calibre.ptempfile import PersistentTemporaryFile, \
-                              PersistentTemporaryDirectory
+from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.date import now as nowf
 
+class LoginFailed(ValueError):
+    pass
+
+class DownloadDenied(ValueError):
+    pass
+
 class BasicNewsRecipe(Recipe):
     '''
     Abstract base class that contains logic needed in all feed fetchers.
@@ -142,11 +147,12 @@ class BasicNewsRecipe(Recipe):
     #: If True empty feeds are removed from the output.
     #: This option has no effect if parse_index is overriden in
     #: the sub class. It is meant only for recipes that return a list
-    #: of feeds using :member:`feeds` or :method:`get_feeds`.
+    #: of feeds using `feeds` or :method:`get_feeds`.
     remove_empty_feeds = False
 
     #: List of regular expressions that determines which links to follow
-    #: If empty, it is ignored. For example::
+    #: If empty, it is ignored. Used only if is_link_wanted is
+    #: not implemented. For example::
     #:
     #:     match_regexps = [r'page=[0-9]+']
     #:
@@ -157,7 +163,8 @@ class BasicNewsRecipe(Recipe):
     match_regexps         = []
 
     #: List of regular expressions that determines which links to ignore
-    #: If empty it is ignored. For example::
+    #: If empty it is ignored. Used only if is_link_wanted is not
+    #: implemented. For example::
     #:
     #:     filter_regexps = [r'ads\.doubleclick\.net']
     #:
@@ -248,9 +255,9 @@ class BasicNewsRecipe(Recipe):
     #: will remove everythong from `<!--Article ends here-->` to `</body>`.
     preprocess_regexps    = []
 
-    #: The CSS that is used to styles the templates, i.e., the navigation bars and
+    #: The CSS that is used to style the templates, i.e., the navigation bars and
     #: the Tables of Contents. Rather than overriding this variable, you should
-    #: use :member:`extra_css` in your recipe to customize look and feel.
+    #: use `extra_css` in your recipe to customize look and feel.
     template_css = u'''
             .article_date {
                 color: gray; font-family: monospace;
@@ -261,14 +268,14 @@ class BasicNewsRecipe(Recipe):
             }
 
             a.article {
-                font-weight: bold;
+                font-weight: bold; text-align:left;
             }
 
             a.feed {
                 font-weight: bold;
             }
 
-            .navbar {
+            .calibre_navbar {
                 font-family:monospace;
             }
     '''
@@ -277,6 +284,15 @@ class BasicNewsRecipe(Recipe):
     #: Override this in your recipe to provide a url to use as a masthead.
     masthead_url = None
 
+    #: By default, the cover image returned by get_cover_url() will be used as
+    #: the cover for the periodical.  Overriding this in your recipe instructs
+    #: calibre to render the downloaded cover into a frame whose width and height
+    #: are expressed as a percentage of the downloaded cover.
+    #: cover_margins = (10,15,'white') pads the cover with a white margin
+    #: 10px on the left and right, 15px on the top and bottom.
+    #: Colors name defined at http://www.imagemagick.org/script/color.php
+    cover_margins = (0,0,'white')
+
     #: Set to a non empty string to disable this recipe
     #: The string will be used as the disabled message
     recipe_disabled = None
@@ -287,6 +303,17 @@ class BasicNewsRecipe(Recipe):
     def short_title(self):
         return self.title
 
+    def is_link_wanted(self, url, tag):
+        '''
+        Return True if the link should be followed or False otherwise. By
+        default, raises NotImplementedError which causes the downloader to
+        ignore it.
+
+        :param url: The URL to be followed
+        :param tag: The Tag from which the URL was derived
+        '''
+        raise NotImplementedError
+
     def get_cover_url(self):
         '''
         Return a :term:`URL` to the cover image for this issue or `None`.
@@ -386,10 +413,26 @@ class BasicNewsRecipe(Recipe):
                     return url
         return article.get('link',  None)
 
+    def skip_ad_pages(self, soup):
+        '''
+        This method is called with the source of each downloaded :term:`HTML` file, before
+        any of the cleanup attributes like remove_tags, keep_only_tags are
+        applied. Note that preprocess_regexps will have already been applied.
+        It is meant to allow the recipe to skip ad pages. If the soup represents
+        an ad page, return the HTML of the real page. Otherwise return
+        None.
+
+        `soup`: A `BeautifulSoup <http://www.crummy.com/software/BeautifulSoup/documentation.html>`_
+        instance containing the downloaded :term:`HTML`.
+        '''
+        return None
+
+
     def preprocess_html(self, soup):
         '''
         This method is called with the source of each downloaded :term:`HTML` file, before
-        it is parsed for links and images.
+        it is parsed for links and images. It is called after the cleanup as
+        specified by remove_tags etc.
         It can be used to do arbitrarily powerful pre-processing on the :term:`HTML`.
         It should return `soup` after processing it.
 
@@ -489,7 +532,7 @@ class BasicNewsRecipe(Recipe):
 
     def get_obfuscated_article(self, url):
         '''
-        If you set :member:`articles_are_obfuscated` this method is called with
+        If you set `articles_are_obfuscated` this method is called with
         every article URL. It should return the path to a file on the filesystem
         that contains the article HTML. That file is processed by the recursive
         HTML fetching engine, so it can contain links to pages/images on the web.
@@ -500,6 +543,19 @@ class BasicNewsRecipe(Recipe):
         '''
         raise NotImplementedError
 
+    def populate_article_metadata(self, article, soup, first):
+        '''
+        Called when each HTML page belonging to article is downloaded.
+        Intended to be used to get article metadata like author/summary/etc.
+        from the parsed HTML (soup).
+        :param article: A object of class :class:`calibre.web.feeds.Article`.
+        If you change the summary, remember to also change the
+        text_summary
+        :param soup: Parsed HTML belonging to this article
+        :param first: True iff the parsed HTML is the first page of the article.
+        '''
+        pass
+
     def postprocess_book(self, oeb, opts, log):
         '''
         Run any needed post processing on the parsed downloaded e-book.
@@ -527,7 +583,10 @@ class BasicNewsRecipe(Recipe):
         self.username = options.username
         self.password = options.password
         self.lrf = options.lrf
-        self.include_navbars = not options.no_inline_navbars
+        self.output_profile = options.output_profile
+        self.touchscreen = getattr(self.output_profile, 'touchscreen', False)
+        if self.touchscreen:
+            self.template_css += self.output_profile.touchscreen_news_css
 
         self.output_dir = os.path.abspath(self.output_dir)
         if options.test:
@@ -572,7 +631,8 @@ class BasicNewsRecipe(Recipe):
 
         self.web2disk_options = web2disk_option_parser().parse_args(web2disk_cmdline)[0]
         for extra in ('keep_only_tags', 'remove_tags', 'preprocess_regexps',
-                      'preprocess_html', 'remove_tags_after', 'remove_tags_before'):
+                      'skip_ad_pages', 'preprocess_html', 'remove_tags_after',
+                      'remove_tags_before', 'is_link_wanted'):
             setattr(self.web2disk_options, extra, getattr(self, extra))
         self.web2disk_options.postprocess_html = self._postprocess_html
         self.web2disk_options.encoding = self.encoding
@@ -580,7 +640,8 @@ class BasicNewsRecipe(Recipe):
         if self.delay > 0:
             self.simultaneous_downloads = 1
 
-        self.navbar = templates.NavBarTemplate()
+        self.navbar = templates.TouchscreenNavBarTemplate() if self.touchscreen else \
+                      templates.NavBarTemplate()
         self.failed_downloads = []
         self.partial_failures = []
 
@@ -599,7 +660,7 @@ class BasicNewsRecipe(Recipe):
         if first_fetch and job_info:
             url, f, a, feed_len = job_info
             body = soup.find('body')
-            if body is not None and self.include_navbars:
+            if body is not None:
                 templ = self.navbar.generate(False, f, a, feed_len,
                                              not self.has_single_feed,
                                              url, __appname__,
@@ -618,15 +679,24 @@ class BasicNewsRecipe(Recipe):
         for attr in self.remove_attributes:
             for x in soup.findAll(attrs={attr:True}):
                 del x[attr]
-        for base in list(soup.findAll('base')):
+        for base in list(soup.findAll(['base', 'iframe'])):
             base.extract()
-        return self.postprocess_html(soup, first_fetch)
+
+        ans = self.postprocess_html(soup, first_fetch)
+        try:
+            article = self.feed_objects[f].articles[a]
+        except:
+            self.log.exception('Failed to get article object for postprocessing')
+            pass
+        else:
+            self.populate_article_metadata(article, ans, first_fetch)
+        return ans
 
 
     def download(self):
         '''
         Download and pre-process all articles from the feeds in this recipe.
-        This method should be called only one on a particular Recipe instance.
+        This method should be called only once on a particular Recipe instance.
         Calling it more than once will lead to undefined behavior.
         @return: Path to index.html
         @rtype: string
@@ -656,7 +726,10 @@ class BasicNewsRecipe(Recipe):
     def feeds2index(self, feeds):
         templ = templates.IndexTemplate()
         css = self.template_css + '\n\n' +(self.extra_css if self.extra_css else '')
-        return templ.generate(self.title, self.timefmt, feeds,
+        timefmt = self.timefmt
+        if self.touchscreen:
+            templ = templates.TouchscreenIndexTemplate()
+        return templ.generate(self.title, "mastheadImage.jpg", timefmt, feeds,
                               extra_css=css).render(doctype='xhtml')
 
     @classmethod
@@ -681,7 +754,8 @@ class BasicNewsRecipe(Recipe):
 
 
 
-    def feed2index(self, feed):
+    def feed2index(self, f, feeds):
+        feed = feeds[f]
         if feed.image_url is not None: # Download feed image
             imgdir = os.path.join(self.output_dir, 'images')
             if not os.path.isdir(imgdir):
@@ -709,7 +783,11 @@ class BasicNewsRecipe(Recipe):
 
         templ = templates.FeedTemplate()
         css = self.template_css + '\n\n' +(self.extra_css if self.extra_css else '')
-        return templ.generate(feed, self.description_limiter,
+
+        if self.touchscreen:
+            templ = templates.TouchscreenFeedTemplate()
+
+        return templ.generate(f, feeds, self.description_limiter,
                               extra_css=css).render(doctype='xhtml')
 
 
@@ -798,8 +876,11 @@ class BasicNewsRecipe(Recipe):
         self.jobs = []
 
         if self.reverse_article_order:
-            feeds = [list(reversed(feed)) for feed in feeds]
+            for feed in feeds:
+                if hasattr(feed, 'reverse'):
+                    feed.reverse()
 
+        self.feed_objects = feeds
         for f, feed in enumerate(feeds):
             feed_dir = os.path.join(self.output_dir, 'feed_%d'%f)
             if not os.path.isdir(feed_dir):
@@ -849,7 +930,7 @@ class BasicNewsRecipe(Recipe):
         #feeds.restore_duplicates()
 
         for f, feed in enumerate(feeds):
-            html = self.feed2index(feed)
+            html = self.feed2index(f,feeds)
             feed_dir = os.path.join(self.output_dir, 'feed_%d'%f)
             with open(os.path.join(feed_dir, 'index.html'), 'wb') as fi:
                 fi.write(html)
@@ -867,10 +948,10 @@ class BasicNewsRecipe(Recipe):
             self.log.error(_('Could not download cover: %s')%str(err))
             self.log.debug(traceback.format_exc())
         if cu is not None:
-            ext = cu.rpartition('.')[-1]
+            ext = cu.split('/')[-1].rpartition('.')[-1]
             if '?' in ext:
                 ext = ''
-            ext = ext.lower() if ext else 'jpg'
+            ext = ext.lower() if ext and '/' not in ext else 'jpg'
             cpath = os.path.join(self.output_dir, 'cover.'+ext)
             if os.access(cu, os.R_OK):
                 with open(cpath, 'wb') as cfile:
@@ -879,6 +960,11 @@ class BasicNewsRecipe(Recipe):
                 self.report_progress(1, _('Downloading cover from %s')%cu)
                 with nested(open(cpath, 'wb'), closing(self.browser.open(cu))) as (cfile, r):
                     cfile.write(r.read())
+                if self.cover_margins[0] or self.cover_margins[1]:
+                    add_borders_to_image(cpath,
+                                         left=self.cover_margins[0],right=self.cover_margins[0],
+                                         top=self.cover_margins[1],bottom=self.cover_margins[1],
+                                         border_color=self.cover_margins[2])
             if ext.lower() == 'pdf':
                 from calibre.ebooks.metadata.pdf import get_metadata
                 stream = open(cpath, 'rb')
@@ -928,63 +1014,18 @@ class BasicNewsRecipe(Recipe):
         '''
         Create a generic cover for recipes that dont have a cover
         '''
-        from calibre.gui2 import is_ok_to_use_qt
-        if not is_ok_to_use_qt():
-            return False
-        img_data = open(I('library.png'), 'rb').read()
-        tdir = PersistentTemporaryDirectory('_default_cover')
-        img = os.path.join(tdir, 'logo.png')
-        with open(img, 'wb') as g:
-            g.write(img_data)
-        img = os.path.basename(img)
-        html= u'''\
-        <html>
-            <head>
-                <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
-                <style type="text/css">
-                    body {
-                        background: white no-repeat fixed center center;
-                        text-align: center;
-                        vertical-align: center;
-                        overflow: hidden;
-                        font-size: 18px;
-                    }
-                    h1 { font-family: serif; }
-                    h2, h4 { font-family: monospace; }
-                </style>
-            </head>
-            <body>
-                <h1>%(title)s</h1>
-                <br/><br/>
-                <div style="position:relative">
-                    <div style="position: absolute; left: 0; top: 0; width:100%%; height:100%%; vertical-align:center">
-                        <img src="%(img)s" alt="calibre" style="opacity:0.3"/>
-                    </div>
-                    <div style="position: absolute; left: 0; top: 0; width:100%%; height:100%%; vertical-align:center">
-                        <h2>%(date)s</h2>
-                        <br/><br/><br/><br/><br/>
-                        <h3>%(author)s</h3>
-                        <br/><br/></br/><br/><br/><br/><br/><br/><br/>
-                        <h4>Produced by %(app)s</h4>
-                    </div>
-                </div>
-            </body>
-        </html>
-        '''%dict(title=self.title if isinstance(self.title, unicode) else self.title.decode(preferred_encoding, 'replace'),
-                 author=self.__author__ if isinstance(self.__author__, unicode) else self.__author__.decode(preferred_encoding, 'replace'),
-                 date=strftime(self.timefmt),
-                 app=__appname__ +' '+__version__,
-                 img=img)
-        hf = os.path.join(tdir, 'cover.htm')
-        with open(hf, 'wb') as f:
-            f.write(html.encode('utf-8'))
-        renderer = render_html(hf)
-        if renderer.tb is not None:
-            self.log.warning('Failed to render default cover')
-            self.log.debug(renderer.tb)
-        else:
-            cover_file.write(renderer.data)
+        try:
+            from calibre.utils.magick_draw import create_cover_page, TextLine
+            title = self.title if isinstance(self.title, unicode) else \
+                    self.title.decode(preferred_encoding, 'replace')
+            date = strftime(self.timefmt)
+            lines = [TextLine(title, 44), TextLine(date, 32)]
+            img_data = create_cover_page(lines, I('library.png'), output_format='jpg')
+            cover_file.write(img_data)
             cover_file.flush()
+        except:
+            self.log.exception('Failed to generate default cover')
+            return False
         return True
 
     def get_masthead_title(self):
@@ -1162,15 +1203,16 @@ class BasicNewsRecipe(Recipe):
                         body = soup.find('body')
                         if body is not None:
                             prefix = '/'.join('..'for i in range(2*len(re.findall(r'link\d+', last))))
-                            if self.include_navbars:
-                                templ = self.navbar.generate(True, num, j, len(f),
-                                                not self.has_single_feed,
-                                                a.orig_url, __appname__, prefix=prefix,
-                                                center=self.center_navbar)
-                                elem = BeautifulSoup(templ.render(doctype='xhtml').decode('utf-8')).find('div')
-                                body.insert(len(body.contents), elem)
+                            templ = self.navbar.generate(True, num, j, len(f),
+                                            not self.has_single_feed,
+                                            a.orig_url, __appname__, prefix=prefix,
+                                            center=self.center_navbar)
+                            elem = BeautifulSoup(templ.render(doctype='xhtml').decode('utf-8')).find('div')
+                            body.insert(len(body.contents), elem)
                             with open(last, 'wb') as fi:
                                 fi.write(unicode(soup).encode('utf-8'))
+        if len(feeds) == 0:
+            raise Exception('All feeds are empty, aborting.')
 
         if len(feeds) > 1:
             for i, f in enumerate(feeds):
@@ -1200,7 +1242,6 @@ class BasicNewsRecipe(Recipe):
         with nested(open(opf_path, 'wb'), open(ncx_path, 'wb')) as (opf_file, ncx_file):
             opf.render(opf_file, ncx_file)
 
-
     def article_downloaded(self, request, result):
         index = os.path.join(os.path.dirname(result[0]), 'index.html')
         if index != result[0]:
@@ -1258,7 +1299,7 @@ class BasicNewsRecipe(Recipe):
                 feed = Feed()
                 msg = 'Failed feed: %s'%(title if title else url)
                 feed.populate_from_preparsed_feed(msg, [])
-                feed.description = unicode(err)
+                feed.description = repr(err)
                 parsed_feeds.append(feed)
                 self.log.exception(msg)
 
@@ -1373,3 +1414,57 @@ class AutomaticNewsRecipe(BasicNewsRecipe):
         if self.use_embedded_content:
             self.web2disk_options.keep_only_tags = []
         return BasicNewsRecipe.fetch_embedded_article(self, article, dir, f, a, num_of_feeds)
+
+class CalibrePeriodical(BasicNewsRecipe):
+
+    #: Set this to the slug for the calibre periodical
+    calibre_periodicals_slug = None
+
+    LOG_IN = 'http://news.calibre-ebook.com/accounts/login'
+    needs_subscription = True
+    __author__ = 'calibre Periodicals'
+
+    def get_browser(self):
+        br = BasicNewsRecipe.get_browser(self)
+        br.open(self.LOG_IN)
+        br.select_form(name='login')
+        br['username'] = self.username
+        br['password'] = self.password
+        raw = br.submit().read()
+        if 'href="/my-account"' not in raw:
+            raise LoginFailed(
+                    _('Failed to log in, check your username and password for'
+                    ' the calibre Periodicals service.'))
+
+        return br
+
+    def download(self):
+        import cStringIO
+        self.log('Fetching downloaded recipe')
+        try:
+            raw = self.browser.open_novisit(
+                'http://news.calibre-ebook.com/subscribed_files/%s/0/temp.downloaded_recipe'
+                % self.calibre_periodicals_slug
+                    ).read()
+        except Exception, e:
+            if hasattr(e, 'getcode') and e.getcode() == 403:
+                raise DownloadDenied(
+                        _('You do not have permission to download this issue.'
+                        ' Either your subscription has expired or you have'
+                        ' exceeded the maximum allowed downloads for today.'))
+            raise
+        f = cStringIO.StringIO(raw)
+        from calibre.utils.zipfile import ZipFile
+        zf = ZipFile(f)
+        zf.extractall()
+        zf.close()
+        from calibre.web.feeds.recipes import compile_recipe
+        from glob import glob
+        try:
+            recipe = compile_recipe(open(glob('*.recipe')[0],
+                'rb').read())
+            self.conversion_options = recipe.conversion_options
+        except:
+            self.log.exception('Failed to compile downloaded recipe')
+        return os.path.abspath('index.html')
+
