@@ -91,6 +91,9 @@ class DummyLock(object):
     def __enter__(self, *args): return self
     def __exit__(self, *args): pass
 
+def default_is_link_wanted(url, tag):
+    raise NotImplementedError()
+
 class RecursiveFetcher(object):
     LINK_FILTER = tuple(re.compile(i, re.IGNORECASE) for i in
                 ('.exe\s*$', '.mp3\s*$', '.ogg\s*$', '^\s*mailto:', '^\s*$'))
@@ -133,7 +136,10 @@ class RecursiveFetcher(object):
         self.remove_tags_before  = getattr(options, 'remove_tags_before', None)
         self.keep_only_tags      = getattr(options, 'keep_only_tags', [])
         self.preprocess_html_ext = getattr(options, 'preprocess_html', lambda soup: soup)
+        self.prepreprocess_html_ext = getattr(options, 'skip_ad_pages', lambda soup: None)
         self.postprocess_html_ext= getattr(options, 'postprocess_html', None)
+        self._is_link_wanted     = getattr(options, 'is_link_wanted',
+                default_is_link_wanted)
         self.download_stylesheets = not options.no_stylesheets
         self.show_progress = True
         self.failed_links = []
@@ -143,8 +149,15 @@ class RecursiveFetcher(object):
         nmassage = copy.copy(BeautifulSoup.MARKUP_MASSAGE)
         nmassage.extend(self.preprocess_regexps)
         nmassage += [(re.compile(r'<!DOCTYPE .+?>', re.DOTALL), lambda m: '')] # Some websites have buggy doctype declarations that mess up beautifulsoup
+        # Remove comments as they can leave detritus when extracting tags leaves
+        # multiple nested comments
+        nmassage.append((re.compile(r'<!--.*?-->', re.DOTALL), lambda m: ''))
         soup = BeautifulSoup(xml_to_unicode(src, self.verbose, strip_encoding_pats=True)[0], markupMassage=nmassage)
 
+        replace = self.prepreprocess_html_ext(soup)
+        if replace is not None:
+            soup = BeautifulSoup(xml_to_unicode(replace, self.verbose, strip_encoding_pats=True)[0], markupMassage=nmassage)
+
         if self.keep_only_tags:
             body = Tag(soup, 'body')
             try:
@@ -207,7 +220,8 @@ class RecursiveFetcher(object):
             if hasattr(err, 'code') and responses.has_key(err.code):
                 raise FetchError, responses[err.code]
             if getattr(err, 'reason', [0])[0] == 104 or \
-                getattr(getattr(err, 'args', [None])[0], 'errno', None) == -2: # Connection reset by peer or Name or service not know
+                getattr(getattr(err, 'args', [None])[0], 'errno', None) in (-2,
+                        -3): # Connection reset by peer or Name or service not know
                 self.log.debug('Temporary error, retrying in 1 second')
                 time.sleep(1)
                 with closing(open_func(url, timeout=self.timeout)) as f:
@@ -233,7 +247,13 @@ class RecursiveFetcher(object):
                 return False
         return True
 
-    def is_link_wanted(self, url):
+    def is_link_wanted(self, url, tag):
+        try:
+            return self._is_link_wanted(url, tag)
+        except NotImplementedError:
+            pass
+        except:
+            return False
         if self.filter_regexps:
             for f in self.filter_regexps:
                 if f.search(url):
@@ -313,6 +333,9 @@ class RecursiveFetcher(object):
                     continue
             try:
                 data = self.fetch_url(iurl)
+                if data == 'GIF89a\x01':
+                    # Skip empty GIF files as PIL errors on them anyway
+                    continue
             except Exception:
                 self.log.exception('Could not fetch image %s'% iurl)
                 continue
@@ -342,7 +365,7 @@ class RecursiveFetcher(object):
         if not self.is_link_ok(iurl):
             self.log.debug('Skipping invalid link:', iurl)
             return None
-        if filter and not self.is_link_wanted(iurl):
+        if filter and not self.is_link_wanted(iurl, tag):
             self.log.debug('Filtered link: '+iurl)
             return None
         return iurl
